/****************************************************************************/
/* ENQRBLD.C -- This file was created with a tab size of 2.                 */
/*                                                                          */
/* This program will rebuild the libraries for you for UltraWin, InTUItion, */
/* and EnCom, as well as rebuild the TUICP.  It is also a neat little demo  */
/* of how InTUItion makes life a bit easier.  This entire program from      */
/* start to finish only took a couple of lazy evenings!                     */
/*                                                                          */
/* As a nifty little feature, if you pass a filename that has the .BAT      */
/* extension on the command line, the program will create for you a batch   */
/* file of all the operations you perform while in the program, rather than */
/* actually do the operations, so that you can construct some nice batch    */
/* files to do more than one rebuild!  For example, if you want to build    */
/* the large, medium, small and compact libraries for Turbo C, you would    */
/* just type "ENQRBLD BLDALL.BAT" on the command line, then select each     */
/* model from the dialog, pressing [Rebuild] for each, and finally [Quit].  */
/* You will then be back at the DOS prompt.  Then just type "BLDALL" (your  */
/* new .BAT file) and go grab a Coke from the fridge.                       */
/*                                                                          */
/* This feature is also necessary for when you don't have enough memory     */
/* for the compiler from the rebuild program, but do have enough from the   */
/* DOS prompt.  The program was tested with DOS 6.0 and 630000 bytes of     */
/* conventional memory, and works for all compilers supported.  If you have */
/* and older version of DOS and a memory hungry compiler, don't panic, just */
/* use this handy feature!                                                  */
/*                                                                          */
/* If you wish to rebuild this rebuild program :), all you have to do is    */
/* compile it under small model, and link with the small model UltraWin     */
/* 2.60 and InTUItion 1.60 libraries.  For Turbo/BorlandC, these would be   */
/* UWTL.LIB and TUITL.LIB.  The size of the program should be around 60K    */
/* when it is all said and done.                                            */
/*                                                         Dr. Boyd Gafford */
/*                                                         EnQue Software   */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>
#include <io.h>
#include <ctype.h>
#ifdef __TURBOC__
#include <dir.h>
#endif
#ifdef M_I86
#include <direct.h>
#endif
#include "t.h"                               /* include the TUI definitions */
#include "ENQRBLD.DEF"

/*----------------------------- project files ------------------------------*/
#define NO_MAKE           0
#define LIBRARY_MAKE      1
#define PROGRAM_MAKE      2

#define NUM_TUI_FILES     12
#define NUM_UW_FILES      20
#define NUM_ENCOM_FILES   6


#define inhibit_item(tp, i)   ((tp)->base[i].flags &= ~(F_SELECTABLE | F_SELECTED | F_TABABLE))
#define uninhibit_item(tp, i) ((tp)->base[i].flags |= (F_SELECTABLE | F_TABABLE))

char *Tui_files[NUM_TUI_FILES] =
{ "t_add",    "t_base", "t_dlg",  "t_fsel", "t_fselt", "t_help",
  "t_helpt",  "t_init", "t_load", "t_save", "t_util", "t_dbg" };

char *Uw_files[NUM_UW_FILES] =
{ "uw_mgr", "uw_vid",   "uw_event", "uw_entry", "uw_menu",
  "uw_ch",  "uw_st",    "uw_prtf",  "uw_rfsh",  "uw_csr",
  "uw_wn",  "uw_ll",    "uw_bdr",   "uw_mask",  "uw_term",
  "uw_font","uw_print", "uw_kbd",   "uw_graph", "uw_dbg" };

char *Encom_files[NUM_ENCOM_FILES] =
{ "en_main", "en_kbd", "en_msg", "en_supp", "en_timer", "en_dbg" };

/*--------------------------- compiler defaults ----------------------------*/
char *Turbo_parms     = "tcc -c -G -O -Z -r -d";
char *Borland_parms   = "bcc -c -G -O -Z -r -d";
char *Msoft_parms     = "cl /c /Gs /Zp /Os";
char *Zortech_parms   = "ztc -c -a -b =9000";
char *Mix_parms       = "pc";
char *Watcom_c_parms  = "wcc386";
char *Watcom_cpp_parms= "wpp386";

/*----------------------------- global TUI ---------------------------------*/
TUI  Main_tui;

int  Num_files;
char **Files;
int  Abort_flag;

FILE *Bat_fp;

/*----------------------------- prototypes ---------------------------------*/
int  main();
void inhibit_model(TUI *tuip);
void allow_model(TUI *tuip);
void do_rebuild_dialog( TUI *tuip );
void do_cmd( char *cmd );
int  ask_question( char *msg );
void show_message( char *msg );
int  register_enqrbld_tui( TUI *tuip );
void register_enqrbld_dlg0( TUI *tuip );
void register_enqrbld_dlg1( TUI *tuip );
void register_enqrbld_dlg2( TUI *tuip );


/*********/
/* ~main */
/*       ********************************************************************/
/* The main function.  Replace the call to load_tui with a call to the      */
/* appropriate register function generated by the TUICP to have your TUI    */
/* linked in.                                                               */
/****************************************************************************/
int main( int argc, char *argv[] )
{
  char fname[81];

  if ( !register_enqrbld_tui(&Main_tui) )
  /* if ( !load_tui(&Main_tui, "ENQRBLD.TUI") ) */
  {
    printf("Sorry, could not register the TUI!\n");
    return(0);
  }
  if (argc == 2)
  {
    strcpy(fname, argv[1]);
    strupr(fname);
    if ( strstr(fname, ".BAT") )
      Bat_fp = fopen(argv[1], "wt");
  }
  init_video(80, 25);
  init_mouse();
  m_show();
  do_rebuild_dialog(&Main_tui);
  m_hide();
  end_mouse();
  end_video();
  free_tui(&Main_tui);
  if (Bat_fp)
    fclose(Bat_fp);
  return(1);
}
/*** end of main ***/

/**********************/
/* ~do_rebuild_dialog */
/*                    *******************************************************/
/* Shell code for typical dialog interaction.  A pointer to the TUI and the */
/* define for the dialog are passed.                                        */
/*                                                                          */
/* Pass the DO_NORMAL define to do_dialog if you do not want to have every  */
/* event the do_dialog function does not understand returned to you!        */
/****************************************************************************/
void do_rebuild_dialog( TUI *tuip)
{
  int   i, dlg_inx = REBUILD_DIALOG, ret_inx, mode, end_flag = OFF;
  char  args[81], cmd[81], libcmd[21], libend[21], libname[21],
        outname[21], ext[5], curr_dir[81], file1[21],
        file2[21], model, path[81], *pathp, obj_ext[11];

  getcwd(curr_dir, 80), strupr(curr_dir);
	if ( strstr(curr_dir, "TUICP") )
		select(tuip, REBUILD_TUICP);
	else if ( strstr(curr_dir, "TUI") )
		select(tuip, REBUILD_TUI);
	else if ( strstr(curr_dir, "UW") )
		select(tuip, REBUILD_UWIN);
	else if ( strstr(curr_dir, "ENCOM") )
		select(tuip, REBUILD_ENCOM);
	else select(tuip, REBUILD_UWIN);

	inhibit_item(tuip, REBUILD_FLAT);
  select(tuip, REBUILD_C);
  select(tuip, REBUILD_LARGE);
  select(tuip, REBUILD_PRODUCTION);
  if ( (pathp = getenv("PATH")) != NULL )
  {
    strcpy(path, pathp), strupr(path);
    if ( strstr(path, "WATCOM") )
    {
      select(tuip, REBUILD_WATCOM);
      inhibit_item(tuip, REBUILD_SMALL);
      inhibit_item(tuip, REBUILD_MEDIUM);
      inhibit_item(tuip, REBUILD_COMPACT);
      inhibit_item(tuip, REBUILD_LARGE);
      uninhibit_item(tuip, REBUILD_FLAT);
      select(tuip, REBUILD_FLAT);
    } else
    if ( strstr(path, "BC") || strstr(path, "BORLANDC") )
      select(tuip, REBUILD_BORLAND);
    else if ( strstr(path, "TC") || strstr(path, "TURBOC") )
      select(tuip, REBUILD_TURBO);
    else if ( strstr(path, "MSC") || strstr(path, "C6") || strstr(path, "C7")
      || strstr(path, "C8") || strstr(path, "MSVC") )
      select(tuip, REBUILD_MSOFT);
    else if ( strstr(path, "ZOR") || strstr(path, "ZTC") )
      select(tuip, REBUILD_ZORTECH);
    else if ( strstr(path, "POWERC") || strstr(path, "MIX") )
      select(tuip, REBUILD_MIX);
    else
      select(tuip, REBUILD_BORLAND);
  }

  center_dialog(tuip, dlg_inx);
  draw_dialog(tuip, dlg_inx);
  while (!end_flag)
  {
    ret_inx = do_dialog(tuip, dlg_inx, DO_NORMAL);
    mode = NO_MAKE;
    Abort_flag = OFF;
    switch( ret_inx )
    {
      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case NO_SELECTION:
        end_flag = ON;
        break;

      case REBUILD_WATCOM:
        inhibit_model(tuip);
        break;

      case REBUILD_TURBO:
        allow_model(tuip);
        break;

      case REBUILD_BORLAND:
        allow_model(tuip);
        break;

      case REBUILD_MIX:
        allow_model(tuip);
        if ( is_selected(tuip, REBUILD_CPP) )
        {
          deselect(tuip, REBUILD_CPP);
          select(tuip, REBUILD_C);
          draw_item(tuip, REBUILD_CPP);
          draw_item(tuip, REBUILD_C);
        }
        break;

      case REBUILD_TUICP:
        allow_model(tuip);
        deselect(tuip, REBUILD_SMALL);
        draw_item(tuip, REBUILD_SMALL);
        deselect(tuip, REBUILD_MEDIUM);
        draw_item(tuip, REBUILD_MEDIUM);
        deselect(tuip, REBUILD_COMPACT);
        draw_item(tuip, REBUILD_COMPACT);
        select(tuip, REBUILD_LARGE);
        draw_item(tuip, REBUILD_LARGE);

      case REBUILD_ZORTECH:
      case REBUILD_MSOFT:
        allow_model(tuip);
        if ( is_selected(tuip, REBUILD_CPP)
          && is_selected(tuip, REBUILD_TUICP) )
        {
          deselect(tuip, REBUILD_CPP);
          select(tuip, REBUILD_C);
          draw_item(tuip, REBUILD_CPP);
          draw_item(tuip, REBUILD_C);
        }
        break;

      case REBUILD_C:
        break;

      case REBUILD_CPP:
        if ( is_selected(tuip, REBUILD_TUICP)
          || is_selected(tuip, REBUILD_MIX) )
        {
          show_message("Only C compiler available!");
          deselect(tuip, REBUILD_CPP);
          select(tuip, REBUILD_C);
          draw_item(tuip, REBUILD_CPP);
          draw_item(tuip, REBUILD_C);
        }
        break;

      case REBUILD_BEGIN:
        deselect(tuip, REBUILD_BEGIN);
        if ( is_selected(tuip, REBUILD_MIX) )
          strcpy(obj_ext, "mix");
        else
          strcpy(obj_ext, "obj");

        /*------------------------- find the product -----------------------*/
        if ( is_selected(tuip, REBUILD_UWIN) )
        {
          Files = Uw_files;
          Num_files = NUM_UW_FILES;
          if ( is_selected(tuip, REBUILD_PRODUCTION) )
            Num_files--;
          strcpy(libname, "uw");
          mode = LIBRARY_MAKE;
        } else
        if ( is_selected(tuip, REBUILD_TUI) )
        {
          Files = Tui_files;
          Num_files = NUM_TUI_FILES;
          if ( is_selected(tuip, REBUILD_PRODUCTION) )
            Num_files--;
          strcpy(libname, "tui");
          mode = LIBRARY_MAKE;
        } else
        if ( is_selected(tuip, REBUILD_ENCOM) )
        {
          Files = Encom_files;
          Num_files = NUM_ENCOM_FILES;
          if ( is_selected(tuip, REBUILD_PRODUCTION) )
            Num_files--;
          strcpy(libname, "en");
          mode = LIBRARY_MAKE;
        } if ( is_selected(tuip, REBUILD_TUICP) )
        {
          strcpy(outname, "tuicp.exe");
          mode = PROGRAM_MAKE;
        }

        /*-------------------------- making a library ----------------------*/
        if (mode == LIBRARY_MAKE)
        {
          /*----------------------- find the extension ---------------------*/
          if ( is_selected(tuip, REBUILD_CPP) )
            strcpy(ext, "cpp");
          else
            strcpy(ext, "c");

          /*-------------------------- find the model ----------------------*/
          if ( is_selected(tuip, REBUILD_SMALL) )
            model = 's';
          else if ( is_selected(tuip, REBUILD_COMPACT) )
            model = 'c';
          else if ( is_selected(tuip, REBUILD_MEDIUM) )
            model = 'm';
          else if ( is_selected(tuip, REBUILD_LARGE) )
            model = 'l';
          else if ( is_selected(tuip, REBUILD_FLAT) )
            model = 'f';

          /*--------------------- do the Turbo parms -----------------------*/
          if ( is_selected(tuip, REBUILD_TURBO) )
          {
            strcpy(libcmd, "tlib");
            strcpy(libend, "");
            strcat(libname, "t");
            if ( is_selected(tuip, REBUILD_CPP) )
              strcat(libname, "cp");
            sprintf(args, "%s -m%c", Turbo_parms, model);
            if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
              strcat(args, " -DENQ_DEBUG");
          } else
          /*-------------------- do the Borland parms ----------------------*/
          if ( is_selected(tuip, REBUILD_BORLAND) )
          {
            strcpy(libcmd, "tlib");
            strcpy(libend, "");
            strcat(libname, "b");
            if ( is_selected(tuip, REBUILD_CPP) )
              strcat(libname, "cp");
            sprintf(args, "%s -m%c", Borland_parms, model);
            if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
              strcat(args, " -DENQ_DEBUG");
          } else
          /*------------------- do the Microsoft parms ---------------------*/
          if ( is_selected(tuip, REBUILD_MSOFT) )
          {
            strcpy(libcmd, "lib");
            strcpy(libend, ";");
            strcat(libname, "m");
            if ( is_selected(tuip, REBUILD_CPP) )
              strcat(libname, "cp");
            sprintf(args, "%s /A%c", Msoft_parms, toupper(model));
            if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
              strcat(args, " /DENQ_DEBUG");
          } else
          /*-------------------- do the Zortech parms ----------------------*/
          if ( is_selected(tuip, REBUILD_ZORTECH) )
          {
            strcpy(libcmd, "zorlib");
            strcpy(libend, ";");
            strcat(libname, "z");
            if ( is_selected(tuip, REBUILD_CPP) )
              strcat(libname, "cp");
            sprintf(args, "%s -m%ci", Zortech_parms, model);
            if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
              strcat(args, " /DENQ_DEBUG");
          } else
          /*----------------------- do the Mix parms -----------------------*/
          if ( is_selected(tuip, REBUILD_MIX) )
          {
            strcpy(libcmd, "merge");
            strcpy(libend, "");
            strcat(libname, "x");
            sprintf(args, "%s /m%c", Mix_parms, model);
            if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
              strcat(args, " /dENQ_DEBUG");
          } else
          /*--------------------- do the Watcom parms ----------------------*/
          if ( is_selected(tuip, REBUILD_WATCOM) )
          {
						strcpy(libcmd, "wlib");
            strcpy(libend, "");
            strcat(libname, "w");
            if ( is_selected(tuip, REBUILD_CPP) )
            {
              strcat(libname, "cp");
              sprintf(args, "%s", Watcom_cpp_parms);
            } else
              sprintf(args, "%s", Watcom_c_parms);
            if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
              strcat(args, " /dENQ_DEBUG");
          }

          sprintf(&libname[strlen(libname)], "%c", model);
          if ( is_selected(tuip, REBUILD_SOURCE_TRACE) )
            strcat(libname, "d");

          sprintf(file1, "%s.c", Files[0]);
          if ( !file_exists(file1) && (Bat_fp == NULL) )
          {
            show_message("Source not in this directory!");
            draw_item(tuip, REBUILD_BEGIN);
            break;
          }
          erase_dialog(tuip, dlg_inx);
          m_hide();
          end_mouse();
          end_video();
          printf("Rebuilding %s.lib, [Space] to pause, [A] to abort.\n\n",
                  libname);

          /*------------------------ kill old lib --------------------------*/
          sprintf(file1, "%s.lib", libname);
          if (Bat_fp)
            sprintf(cmd, "del %s", file1), do_cmd(cmd);
          else if ( file_exists(file1) )
            unlink(file1);

          /*----------------------- rename to cpp --------------------------*/
          if ( is_selected(tuip, REBUILD_CPP) )
            for (i=0; i<Num_files; i++)
            {
              sprintf(file1, "%s.c", Files[i]);
              sprintf(file2, "%s.%s", Files[i], ext);
              if (Bat_fp)
                sprintf(cmd, "rename %s %s", file1, file2), do_cmd(cmd);
              else if ( file_exists(file1) )
                rename(file1, file2);
            }

          /*---------------------- rebuild the lib -------------------------*/
          for (i=0; i<Num_files; i++)
          {
            if ( !Abort_flag )
              sprintf(cmd, "%s %s.%s", args, Files[i], ext), do_cmd(cmd);
            if ( !Abort_flag )
            {
              if ( is_selected(tuip, REBUILD_MIX) )
                sprintf(cmd, "%s %s.lib %s.mix",
                        libcmd, libname, Files[i]), do_cmd(cmd);
              else
              if( !i && is_selected(tuip, REBUILD_ZORTECH) )
                sprintf(cmd, "%s %s.lib/C +%s.obj%s",
                        libcmd, libname, Files[i], libend), do_cmd(cmd);
              else
                sprintf(cmd, "%s %s.lib +%s.obj%s",
                        libcmd, libname, Files[i], libend), do_cmd(cmd);
            }
            sprintf(file1, "%s.%s", Files[i], obj_ext);
            if (Bat_fp)
              sprintf(cmd, "del %s", file1), do_cmd(cmd);
            else if ( file_exists(file1) )
              unlink(file1);
          }

          /*------------------------- back to c ----------------------------*/
          if ( is_selected(tuip, REBUILD_CPP) )
            for (i=0; i<Num_files; i++)
            {
              sprintf(file1, "%s.%s", Files[i], ext);
              sprintf(file2, "%s.c", Files[i]);
              if (Bat_fp)
                sprintf(cmd, "rename %s %s", file1, file2), do_cmd(cmd);
              else if ( file_exists(file1) )
                rename(file1, file2);
            }

          /*------------------------- kill bak file ------------------------*/
          sprintf(file1, "%s.bak", libname);
          if (Bat_fp)
            sprintf(cmd, "del %s", file1), do_cmd(cmd);
          else if ( file_exists(file1) )
            unlink(file1);

          /*----------------------- kill unfinished lib --------------------*/
          if (Abort_flag)
          {
            sprintf(file1, "%s.lib", libname);
            if (Bat_fp)
              sprintf(cmd, "del %s", file1), do_cmd(cmd);
            else if ( file_exists(file1) )
              unlink(file1);
          }
          init_video(80, 25);
          init_mouse();
          m_show();
          draw_dialog(tuip, dlg_inx);
        } else
        /*-------------------------- making the TUICP ----------------------*/
        {
          if ( !file_exists("tuicp_1.c") )
          {
            show_message("Source not in this directory!");
            draw_item(tuip, REBUILD_BEGIN);
            break;
          }
          erase_dialog(tuip, dlg_inx);
          m_hide();
          end_mouse();
          end_video();
          printf("Rebuilding tuicp.exe\n\n");

          if ( is_selected(tuip, REBUILD_MSOFT) )
            do_cmd("cl /Gs /Zp /Os /AL tuicp_?.c /Fetuicp.exe /link /NOE /ST:8192 uwml.lib tuiml.lib");
          else
          if ( is_selected(tuip, REBUILD_TURBO) )
            do_cmd("tcc -G -O -Z -r -d -ml -r -etuicp.exe tuicp_?.c uwtl.lib tuitl.lib");
          else
          if ( is_selected(tuip, REBUILD_BORLAND) )
            do_cmd("bcc -G -O -Z -r -d -ml -r -etuicp.exe tuicp_?.c uwbl.lib tuibl.lib");
          else
          if ( is_selected(tuip, REBUILD_ZORTECH) )
            do_cmd("ztc -a -b =9000 -mli tuicp_?.c uwzl.lib tuizl.lib -otuicp.exe");
          else
          if ( is_selected(tuip, REBUILD_WATCOM) )
            do_cmd("wcl386 tuicp_?.c /k8192 /l=dos4g /fe=tuicp.exe uwwf.lib tuiwf.lib");
          else
          if ( is_selected(tuip, REBUILD_MIX) )
          {
            do_cmd("pc /ml tuicp_?.c");
            do_cmd("pcl tuicp_1 tuicp_2 tuicp_3 tuicp_4 tuicp_5 tuicp_6 tuicp_7 tuicp_8 tuicp_9 ;uwxl.lib ;tuixl.lib");
            do_cmd("copy tuicp_1.exe tuicp.exe");
            if ( Bat_fp )
              do_cmd("del tuicp_1.exe");
            else if ( file_exists("tuicp_1.exe") )
              unlink("tuicp_1.exe");
          }
          init_video(80, 25);
          init_mouse();
          m_show();
          draw_dialog(tuip, dlg_inx);
        }
        break;

      case REBUILD_QUIT:
        deselect(tuip, ret_inx);
        end_flag = ON;
        break;
    }
  }
  erase_dialog(tuip, dlg_inx);
}
/*** end of do_rebuild_dialog ***/

/******************/
/* ~inhibit_model */
/*                ***********************************************************/
/****************************************************************************/
void inhibit_model(TUI *tuip)
{
  inhibit_item(tuip, REBUILD_SMALL);
  draw_item(tuip, REBUILD_SMALL);
  inhibit_item(tuip, REBUILD_MEDIUM);
  draw_item(tuip, REBUILD_MEDIUM);
  inhibit_item(tuip, REBUILD_COMPACT);
  draw_item(tuip, REBUILD_COMPACT);
  inhibit_item(tuip, REBUILD_LARGE);
  draw_item(tuip, REBUILD_LARGE);
  select(tuip, REBUILD_FLAT);
  draw_item(tuip, REBUILD_FLAT);

  uninhibit_item(tuip, REBUILD_FLAT);
  select(tuip, REBUILD_FLAT);
  draw_item(tuip, REBUILD_FLAT);
}
/*** end of inhibit_model ***/

/****************/
/* ~allow_model */
/*              *************************************************************/
/****************************************************************************/
void allow_model(TUI *tuip)
{
  uninhibit_item(tuip, REBUILD_SMALL);
  draw_item(tuip, REBUILD_SMALL);
  uninhibit_item(tuip, REBUILD_MEDIUM);
  draw_item(tuip, REBUILD_MEDIUM);
  uninhibit_item(tuip, REBUILD_COMPACT);
  draw_item(tuip, REBUILD_COMPACT);
  uninhibit_item(tuip, REBUILD_LARGE);
  draw_item(tuip, REBUILD_LARGE);
  if ( is_selected(tuip, REBUILD_FLAT) )
  {
    inhibit_item(tuip, REBUILD_FLAT);
    draw_item(tuip, REBUILD_FLAT);
    select(tuip, REBUILD_LARGE);
    draw_item(tuip, REBUILD_LARGE);
  }
}
/*** end of inhibit_model ***/

/***********/
/* ~do_cmd */
/*         ******************************************************************/
/* Call the system command.                                                 */
/****************************************************************************/
void do_cmd( char *cmd )
{
  if (!Abort_flag)
  {
    printf("%s\n", cmd);
    if (Bat_fp == NULL)
      system(cmd);
    else
      fprintf(Bat_fp, "%s\n", cmd);
  }
  if ( check_key() )
  {
    switch(get_key())
    {
      case ' ':
        printf("Paused, [A] to abort, any other key to continue...");
        wait_event();
        printf("\n");
        if (!Event.is_mouse)
          if ( (Event.key == 'A') || (Event.key == 'a'))
            Abort_flag = ON;
        break;
      case 'A': case 'a':
        Abort_flag = ON;
        break;
    }
  }
}
/*** end of do_cmd ***/

/*****************/
/* ~ask_question */
/*               ************************************************************/
/* Pull up the question dialog box and get input from the user!             */
/****************************************************************************/
int ask_question( char *msg )
{
  TUI *tuip = &Main_tui;
  WINDOW *wnp;
  int inx, stat = 0, end_flag = 0;

  center_dialog(tuip, ASK_DIALOG);
  draw_dialog(tuip, ASK_DIALOG);
  wnp = get_dlg_wnp(tuip, ASK_DIALOG);
  wn_plst(CENTERED, 1, msg, wnp);
  while( !end_flag )
  {
    inx = do_dialog(tuip, ASK_DIALOG, DO_NORMAL);
    switch(inx)
    {
      /*------------------------ cancel pressed ----------------------------*/
      case ASK_CANCEL:
        deselect(tuip, inx);

      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case NO_SELECTION:
        end_flag = ON;
        break;

      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case ASK_OK:
        deselect(tuip, inx);
        stat = end_flag = 1;
        break;
    }
  }
  erase_dialog(tuip, ASK_DIALOG);
  return(stat);
}
/*** end of ask_question ***/

/*****************/
/* ~show_message */
/*               ************************************************************/
/* Pull up the question dialog box and get input from the user!             */
/****************************************************************************/
void show_message( char *msg )
{
  TUI     *tuip = &Main_tui;
  WINDOW  *wnp;

  deselect(tuip, MESSAGE_OK);
  center_dialog(tuip, MESSAGE_DIALOG);
  draw_dialog(tuip, MESSAGE_DIALOG);
  wnp = get_dlg_wnp(tuip, MESSAGE_DIALOG);
  wn_plst(CENTERED, 1, msg, wnp);
  do_dialog(tuip, MESSAGE_DIALOG, DO_NORMAL);
  erase_dialog(tuip, MESSAGE_DIALOG);
}
/*** end of ask_question ***/

/*------------------- slider/image/palette globals -------------------------*/
uchar Palette_enqrbld[16] =
  { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x14, 0x07,
    0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F };

/*----------------------------- prototypes ---------------------------------*/
int  register_enqrbld_tui( TUI *tuip );
void register_enqrbld_dlg0( TUI *tuip );
void register_enqrbld_dlg1( TUI *tuip );
void register_enqrbld_dlg2( TUI *tuip );

/*************************/
/* ~register_enqrbld_tui */
/*                       ****************************************************/
/* Call this function to register the TUI.  This replaces load_tui!         */
/****************************************************************************/
int register_enqrbld_tui( TUI *tuip )
{
  Tui_error = OFF;
  setmem(tuip, sizeof(TUI), 0);
  tuip->version_num = 160L;
  tuip->blink_set = ON;
  tuip->rows = 25;
  tuip->back_att = 0x03;
  tuip->back_bdr_att = 0x03;
  movmem(Palette_enqrbld, tuip->palette, 16);

  register_enqrbld_dlg0(tuip);
  register_enqrbld_dlg1(tuip);
  register_enqrbld_dlg2(tuip);
  set_tui_parenting(tuip);
  if (Tui_error)
  {
    free_tui(tuip);
    return(0);
  } else return(1);
}
/*** end of register_enqrbld_tui ***/

/**************************/
/* ~register_enqrbld_dlg0 */
/*                        ***************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_enqrbld_dlg0( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 0, 1, 80, 23, 0x70, 0x71, 3);
  add_button(tuip, 0, 0x0000, 0, 21, 0, 37, 1, 0,
    0x71, 0x71, 0x71, 0x71, 0x71, 0, "\335 EnQue Software's Rebuild Facility \336");
  add_box(tuip, 0, 0x0000, 0, 3, 2, 73, 3, 0x10, 1);
  add_box(tuip, 0, 0x0000, 0, 3, 6, 31, 5, 0x1F, 1);
  add_box(tuip, 0, 0x0000, 0, 53, 6, 23, 13, 0x1F, 1);
  add_box(tuip, 0, 0x0000, 0, 37, 6, 14, 5, 0x1F, 1);
  add_box(tuip, 0, 0x0000, 0, 3, 12, 48, 7, 0x1F, 1);
  add_button(tuip, 0, 0x0000, 0, 5, 2, 8, 1, 0,
    0x1F, 0x1f, 0x1f, 0x1f, 0x1f, 0, "Product:");
  add_check(tuip, 87, 0x1007, 0, 5, 3, 16, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 10, " [ ] UltraWin");
  add_check(tuip, 73, 0x1007, 0, 23, 3, 16, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] InTUItion");
  add_check(tuip, 84, 0x1017, 0, 41, 3, 15, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] TUICP");
  add_check(tuip, 69, 0x1007, 0, 58, 3, 15, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] EnCom");
  add_button(tuip, 0, 0x0000, 0, 5, 6, 8, 1, 0,
    0x1F, 0x1f, 0x1f, 0x1f, 0x1f, 0, "Options:");
  add_button(tuip, 0, 0x0000, 0, 39, 6, 6, 1, 0,
    0x1F, 0x1f, 0x1f, 0x1f, 0x1f, 0, "Under:");
  add_button(tuip, 0, 0x0000, 0, 55, 6, 9, 1, 0,
    0x1F, 0x1f, 0x1f, 0x1f, 0x1f, 0, "Compiler:");
  add_check(tuip, 83, 0x1007, 0, 5, 7, 26, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Source Trace Version");
  add_check(tuip, 67, 0x1017, 0, 39, 7, 9, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] C");
  add_check(tuip, 51, 0x1017, 0, 55, 7, 18, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 13, " [ ] Watcom C32");
  add_check(tuip, 80, 0x1007, 0, 5, 9, 26, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Production Version");
  add_check(tuip, 43, 0x1017, 0, 39, 9, 9, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 7, " [ ] C++");
  add_check(tuip, 85, 0x1017, 0, 55, 9, 18, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 6, " [ ] Turbo C");
  add_check(tuip, 66, 0x1017, 0, 55, 11, 18, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Borland C");
  add_button(tuip, 0, 0x0000, 0, 5, 12, 7, 1, 0,
    0x1F, 0x1f, 0x1f, 0x1f, 0x1f, 0, "Models:");
  add_check(tuip, 65, 0x1017, 0, 5, 13, 19, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 7, " [ ] Small");
  add_check(tuip, 79, 0x1017, 0, 29, 13, 19, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 6, " [ ] Compact");
  add_check(tuip, 77, 0x1017, 0, 55, 13, 18, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Microsoft C");
  add_check(tuip, 68, 0x1017, 0, 5, 15, 19, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 7, " [ ] Medium");
  add_check(tuip, 76, 0x1017, 0, 29, 15, 19, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Large");
  add_check(tuip, 90, 0x1017, 0, 55, 15, 18, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Zortech C");
  add_check(tuip, 70, 0x1017, 0, 5, 17, 19, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 5, " [ ] Flat");
  add_check(tuip, 88, 0x1017, 0, 55, 17, 18, 1, 1,
    0x70, 0x7F, 0x0e, 2, 0xFB, 7, " [ ] Mix C");
  add_button(tuip, 82, 0x0017, 0, 3, 20, 34, 1, 1,
    0x17, 0x1f, 0x07, 0x0f, 0x0e, 0, "Rebuild!");
  add_button(tuip, 81, 0x0017, 0, 42, 20, 34, 1, 1,
    0x17, 0x1f, 0x07, 0x0f, 0x0e, 0, "Quit");
}
/*** end of register_enqrbld_dlg0 ***/

/**************************/
/* ~register_enqrbld_dlg1 */
/*                        ***************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_enqrbld_dlg1( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 17, 9, 43, 7, 0x4E, 0x40, 3);
  add_button(tuip, 67, 0x0017, 0, 2, 4, 18, 1, 1,
    0x70, 0x7f, 0x04, 0x04, 0x04, 0, "Cancel");
  add_button(tuip, 75, 0x0017, 0, 22, 4, 18, 1, 1,
    0x70, 0x7f, 0x04, 0x04, 0x04, 1, "Ok");
}
/*** end of register_enqrbld_dlg1 ***/

/**************************/
/* ~register_enqrbld_dlg2 */
/*                        ***************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_enqrbld_dlg2( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 17, 9, 43, 7, 0x4E, 0x40, 3);
  add_button(tuip, 75, 0x0017, 0, 12, 4, 18, 1, 1,
    0x70, 0x7f, 0x04, 0x04, 0x04, 1, "Ok");
}
/*** end of register_enqrbld_dlg2 ***/

/*** END OF FILE ***/