/****************************************************************************/
/* G_CH.C                                                                   */
/*                                                                          */
/*  This routine is provided as a sample of a fast graphics funtion.  It    */
/* outputs one character to the screen under EGA 640x350 and VGA 640x480    */
/* 16-color modes.  It has some inline assembly, so any version of Turbo    */
/* or Borland C with TASM.EXE available or any version of Microsoft C above */
/* 5.00 is needed to compile.  This routine is provided as a sample.  It    */
/* would be very easy to make this a stub function that simply calls a      */
/* function from an external graphics library to output one character.      */
/*                                                                          */
/*  Please feel free to use this in any of your programs.                   */
/*                                                                          */
/*                                                          Kevin Huck      */
/*                                                          Boyd Gafford    */
/****************************************************************************/

#ifdef M_I86
#define asm __asm
#endif
#ifdef __TURBOC__
#pragma inline
#endif

/*********/
/* ~g_ch */
/*       ********************************************************************/
/*  This routine writes a character "c" at "x", "y" on an EGA/VGA graphics  */
/*  screen...                                                               */
/*  IMPORTANT: Global data cannot be accessed in this routine past the      */
/*    marked area as the ds register is used to point to the video buffer...*/
/****************************************************************************/
void g_ch( int c, int r, uchar v )
{
  uchar far *s;
  int  bk_flag = 0, fg_flag = 0, bpr;
  bpr = Bytes_per_row;

  if( G_opt )
  {
    s = Screen + (r * (V_cols * 2)) + (c * 2);
    if( *s++ == (uchar) v )
    {
      if( *s == (uchar) ((Bkgnd << 4) | Fgnd) )
        return;
      if( (uchar) (*s & 0x0f) == Fgnd )
        bk_flag = 1;
      if( (uchar) (*s >> 4) == Bkgnd )
        fg_flag = 1;
    }
  }
  c = c * 8;                      /* font must always be 8 bits wide        */
  r = r * Font_spacing;           /* font heigth is variable                */

#if defined(__TURBOC__) || defined(M_I86)
  asm push di
  asm push si
  asm push ds

/*-------------------- calculate first pixel address -----------------------*/

  asm mov  ax,r                   /* y address                              */
  asm mov  bx,c                   /* x address                              */

  asm mov  cl,bl                  /* cl = low-order byte of x               */
  asm push dx                     /* preserve dx                            */
  asm mov  dx,80                  /* ax = y * bytes per line                */
  asm mul  dx
  asm pop  dx

  asm shr  bx,1
  asm shr  bx,1
  asm shr  bx,1                   /* bx = x/8                               */
  asm add  bx,ax                  /* bx = y*bytes per line + x/8            */
  asm add  bx,Goff                /* add byte offset into video buffer      */
  asm mov  ax,Gseg                /* get segment for video buffer           */

  asm mov  es,ax                  /* es:bx = byte address of pixel          */

  asm push es                     /* preserve video buffer segment          */
  asm mov  si,bx                  /* si = video buffer offset               */

  asm mov  cx,Font_rows           /* cx = scan lines for current font       */
  asm mov  bl,Fgnd                /* get foreground and background colors   */
  asm mov  bh,Bkgnd
  asm push bx                     /* save them for later                    */

/*-------------- set up character defintion table addressing ---------------*/
/*-------------- CANNNOT USED GLOBAL DATA PAST THIS POINT ------------------*/

  asm xor  ax,ax
  asm mov  ds,ax                  /* ds = absolute zero                     */

  asm mov  al,v                   /* al = character code                    */
  asm mov  bx,43h*4               /* ds:bx -> int 43h vector                */
  asm les  di,ds:[bx]             /* es:di -> start of character table      */
  asm mul  cl                     /* ax = offset into char def table        */
                                  /* (Font_rows X char code)                */
  asm add  di,ax                  /* di = address of char def               */

  asm pop  bx                     /* get foreground, background colors      */
  asm pop  ds                     /* ds:si -> video buffer (from ds)        */

/*----------------- set up Graphics Controller registers -------------------*/

  asm mov  dx,3CEh                /* Graphics Controller address reg port   */

  asm mov  ax,0A05h               /* al = Mode register number              */
                                  /* ah = Write Mode 2 (bits 0-1)           */
                                  /*      Read  Mode 1 (bit  4)             */
  asm out  dx,ax

  asm mov  ah,18h                 /* ah = Read-Modify-Write bits            */
  asm mov  al,0                   /* al = Data Rotate/Funtion Select reg    */
  asm out  dx,ax

  asm mov  ax,0007                /* ah = Color Don't Care bits             */
                                  /* al = Color Don't Cate reg number       */
  asm out  dx,ax                  /* "don't care" for all bit planes        */


/*---------------------- output byte aligned character ---------------------*/
  asm shr  cx,1                   /* output two bytes per loop for speed    */
  asm mov  al,8                   /* al = Bit Mask register number          */

  asm out  dx,al
  asm inc  dx

  asm cmp  word ptr bk_flag,1
  asm je   l1
  asm cmp  word ptr fg_flag,1
  asm je   l2
  asm jmp  l3

/*---------------------------- background only -----------------------------*/
l1:
  asm mov  ax,es:[di]             /* ah = pattern for next row of pixels    */
  asm add  di,2                   /* es:di -> next word in char def table   */

  asm not  al
  asm out  dx,al
  asm and  [si],bh                /* update background pixels               */
  asm add  si,bpr

  asm mov  al,ah

  asm not  al
  asm out  dx,al
  asm and  [si],bh                /* update background pixels               */
  asm add  si,bpr
  asm loop l1

  asm jmp  egaend


/*----------------------------- foreground only ----------------------------*/
l2:
  asm mov  ax,es:[di]             /* ah = pattern for next row of pixels    */
  asm add  di,2                   /* es:di -> next word in char def table   */

  asm out  dx,al                  /* update Bit Mask register               */
  asm and  [si],bl                /* update foreground pixels               */
  asm add  si,bpr

  asm mov  al,ah

  asm out  dx,al                  /* update Bit Mask register               */
  asm and  [si],bl                /* update foreground pixels               */
  asm add  si,bpr
  asm loop l2

  asm jmp egaend

/*-------------------------- background & foreground -----------------------*/
l3:
  asm mov  ax,es:[di]             /* ah = pattern for next row of pixels    */
  asm add  di,2                   /* es:di -> next word in char def table   */

  asm cmp  al,0
  asm je   bkgnd1
  asm out  dx,al                  /* update Bit Mask register               */
  asm and  [si],bl                /* update foreground pixels               */
bkgnd1:
  asm not  al
  asm out  dx,al
  asm and  [si],bh                /* update background pixels               */
  asm add  si,bpr

  asm mov  al,ah

  asm cmp  al,0
  asm je   bkgnd2
  asm out  dx,al                  /* update Bit Mask register               */
  asm and  [si],bl                /* update foreground pixels               */
bkgnd2:
  asm not  al
  asm out  dx,al
  asm and  [si],bh                /* update background pixels               */
  asm add  si,bpr
  asm loop l3

egaend:
  asm pop  ds
  asm pop  si
  asm pop  di
#endif
  return;
}
/*** end of g_ch **/

/*** END OF FILE ***/
