/****************************************************************************/
/* PLAYDEMO.C                                                               */
/*                                                                          */
/* This file is a nifty little module for playing back demo files created   */
/* with our new "automatic tutorial" capability, now a part of the TUICP.   */
/* The .TUT files we distribute are actually ASCII text files that contain  */
/* a recording of a session under the TUICP, that can be also be played     */
/* back with this little stand-alone program.  This program uses UltraWin   */
/* for some simple popup windows and for finer control over the PC timer.   */
/*                                                                          */
/* To use this program, simply type PLAYDEMO FILENAME.EXT. For example,     */
/* PLAYDEMO ENTRY.TUT will play the TUICP data entry demonstration.  You    */
/* are more than welcome to utilize this capability in your own programs.   */
/* Refer to TUICP_9.C for how to include this capability into your own      */
/* programs, and you'll be doing these neat demos in no time at all!        */
/*                                                                          */
/* DEMO FILE INFORMATION:                                                   */
/* The data in the file is simply screen changes recorded at a sampling     */
/* rate of 15/sec.  Each change (what we call a "delta") is recorded by     */
/* storing the offset into the screen buffer, the length of the change, and */
/* finally the char/att pairs (one per usint) that make the change.         */
/* Times during which no changes occur are recorded as 0xfffa and the       */
/* number of tics (91 per second).  During the tutorial, "chat windows" can */
/* be used to "talk" to the user at any time during playback.  These are    */
/* recorded by using 0xfffb followed by the number of lines for the window, */
/* followed by the lines of ASCII text.                                     */
/*                                                                          */
/* The data that is written to disk is in the form of text lines that are   */
/* composed of hex integers (4 ASCII chars per) followed by a CR/LF.  This  */
/* makes it easy to pull it into a text editor and modify it!               */
/* The hex characters are 0-9 and a-f (lowercase) to make parsing quick.    */
/* The form of the lines are as follows:                                    */
/*                                                                          */
/* usint    usint   usint                                                   */
/* OFFSET   LENGTH  DATA...            screen changes (both char and att)   */
/* 0xfffa   TICS                       for waiting with no changes          */
/* 0xfffb   LINES                      for popup window, next LINES in the  */
/*                                     text file to be displayed in a popup */
/*                                     window followed by wait for [Enter]. */
/*                                                                          */
/* You will note that even though the files get pretty big, that they ZIP   */
/* incredibly well, most of the time over 90% -- great for distribution!    */
/*                                                                          */
/*                                                          Boyd Gafford    */
/*                                                          05/01/94        */
/****************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "uw.h"

#define  DEMO_DELAY   0xfffa
#define  DEMO_CHAT    0xfffb

/*-------------------------- globals for module ----------------------------*/
FILE    *Demo_fp;                           /* file for saving data         */
long    Playback_pct = 100L;

/*---------------------------- prototypes ----------------------------------*/
int  main( int argc, char *argv[] );
int  playback_recording( void );
int  display_chat_text( usint lines );

/*********/
/* ~main */
/*       ********************************************************************/
/****************************************************************************/
int main( int argc, char *argv[] )
{
  if ( argc == 2 )
  {
    if ( (Demo_fp = fopen(argv[1], "rt")) != NULL )
    {
      init_video(80, 25);
      init_clock(0x3333);
      playback_recording();
      end_clock();
      end_video();
    } else printf("Could not find the file [%s].\n", argv[1]);
  } else printf("PLAYDEMO needs the name of the file to play!\n");
  return(1);
}
/*** end of main ***/

/**********************/
/* ~display_chat_text */
/*                    *******************************************************/
/****************************************************************************/
void strip_line( char *s )
{
  char *s1 = s;
  while( *s1 ) s1++;
  s1--;
  while( isspace(*s1) ) *s1-- = '\0';
}
int display_chat_text( usint lines )
{
  WINDOW wn, *wnp = &wn;
  int x, y, r = 1, save_lines = lines, stat = 1;
  char buff[133];

  rd_csr( &x, &y );
  wn_create( 10, V_rows/2 - lines/2 - 2, 70, V_rows/2 + lines/2 + 3,
             DBL_BDR, WN_POPUP, wnp );
  wn_color( LIGHTGRAY, BLACK, wnp );
  wn_bdr_color( LIGHTGRAY, BLACK, wnp );
  wn_set(wnp);
  push(wnp->inside), wnp->inside = OFF;
  wn_plst(CENTERED, 0, "[ Chat Window ]", wnp);
  pop(wnp->inside);
  while(lines-- > 0)
  {
    if( fgets(buff, 80, Demo_fp) == NULL )
      break;
    strip_line(buff);
    wn_plst( 1, r, buff, wnp );
    if( r < wn_rqty(wnp) - 2 )
      r++;
  }
  wn_color(RED, BLACK, wnp);
  wn_plst(CENTERED, wn_rqty(wnp) - 1, "Press [Enter] to continue...", wnp);
  Uw_timers[2] = (int) (((long) (91 * (save_lines * 4)) * Playback_pct) / 100L);
  while( Uw_timers[2] > 0 )
    if( event_pending() )
      if ( !Event.is_mouse )
      {
        if ( Event.key == KEY_ENTER )
          break;
        else
        if ( Event.key == KEY_ESC )
        {
          stat = 0;
          break;
        } else
        switch(Event.key)
        {
          case '-':
            Playback_pct += 5L;
            break;
          case '+':
            if ( Playback_pct >= 20 )
              Playback_pct -= 5;
            break;
        }
      }
  mv_csr(x, y);
  wn_destroy(wnp);
  return(stat);
}
/*** end of display_chat_text ***/

/**************/
/* ~get_usint */
/*            ***************************************************************/
/* This routine takes 4 ASCII characters (like fffa) and converts it into   */
/* an unsigned short integer (two bytes).  The demo files are simply a      */
/* series of hex values in a text file!                                     */
/****************************************************************************/
usint get_usint( char *s )
{
  usint val = 0;

  val |= (isdigit(*s)) ? *s++ - '0' : *s++ - 'W';
  val <<= 4;
  val |= (isdigit(*s)) ? *s++ - '0' : *s++ - 'W';
  val <<= 4;
  val |= (isdigit(*s)) ? *s++ - '0' : *s++ - 'W';
  val <<= 4;
  val |= (isdigit(*s)) ? *s++ - '0' : *s++ - 'W';
  return(val);
}
/*** end of get_usint ***/

/***********************/
/* ~playback_recording */
/*                     ******************************************************/
/****************************************************************************/
int playback_recording( void )
{
  usint offset, len, *save_area;
  usint UW_FAR  *vid_area = (usint UW_FAR *) Screen;
  int   i, status = 1, end_flag = OFF;
  char  buffer[350], *buffp;

  /*-------------------- first save the current screen ---------------------*/
  save_area = calloc(80*25, sizeof(usint));
  if (save_area == NULL)
    return(0);
  for (i=0; i<2000; i++)
    save_area[i] = vid_area[i];

  while ( !end_flag && (fgets(buffer, 350, Demo_fp) != NULL) )
  {
    buffp = buffer;
    offset = get_usint(buffp), buffp += 4;
    len = get_usint(buffp), buffp += 4;
    /*------------------------- is it a delay? ---------------------------*/
    if ( offset == DEMO_DELAY )
    {
      Uw_timers[0] = (int) (((long) len * Playback_pct) / 100L);
      while ( Uw_timers[0] > 0 )
        if ( event_pending() )
          if ( !Event.is_mouse )
            switch( Event.key )
            {
              case '-':
                Playback_pct += 5L;
                break;
              case '+':
                if ( Playback_pct >= 20 )
                  Playback_pct -= 5;
                break;
              case KEY_ESC:
                Uw_timers[0] = 1;
                end_flag = ON, status = 0;
                break;
            }
    } else
    /*----------------------- it is a chat window ------------------------*/
    if ( offset == DEMO_CHAT )
    {
      if ( !display_chat_text(len) )
        end_flag = ON, status = 0;
    } else
    /*------------------------- it is a delta ----------------------------*/
      while (len-- > 0)
        vid_area[offset++] = get_usint(buffp), buffp += 4;
  }

  /*-------------------- now restore the current screen --------------------*/
  for (i=0; i<2000; i++)
    vid_area[i] = save_area[i];
  free(save_area);
  return(status);
}
/*** end of playback_recording ***/

/**** END OF FILE ****/
