/****************************************************************************/
/*                                                                          */
/* TD_MAIN.C                                                                */
/*                                                                          */
/* This program is a fully functioning terminal program, complete with ANSI */
/* terminal emulation (without ANSI.SYS) to a window!  Full source code to  */
/* the terminal code is included.  This program uses UltraWin 2.10 and      */
/* InTUItion 1.10 to handle the windowing and user-interface part of the    */
/* program, and serves as an excellent example of how UltraWin, InTUItion,  */
/* and the EnCom libraries can make life easier for the C programmer.       */
/*                                                                          */
/*                                                      Dr. Boyd G. Gafford */
/*                                                      EnQue Software      */
/*                                                      03/20/92            */
/*                                                                          */
/****************************************************************************/
#if defined(M_I86) || defined(__WATCOMC__)
  #include <direct.h>
#else
  #include <dir.h>
#endif
#include <io.h>
#include "t.h"                                  /* the main TUI header file */
#include "encom.h"                              /* the Encom header file    */

#include "td.h"                                 /* our mini-app header file */
#include "td.def"                               /* the TUICP define file    */
#include "td_glob.h"                            /* globals for this app     */

/*********/
/* ~main */
/*       ********************************************************************/
/* This is the mail loop that does the initialization, then does the key    */
/* and mouse processing, and finally does cleanup before exiting to DOS.    */
/****************************************************************************/
void main( int argc, char *argv[] )
{
  TUI   *tuip = &Encom_tui;
  int   end_flag = OFF;
  TERM  *termp;
  PORT  *portp;
  XFER  *xferp;
  int   port = COM2;
  long  baud = 9600L;

  if( argc >= 2 )
  {
    switch(argv[1][0])
    {
      case '1': port = COM1; break;
      case '2': port = COM2; break;
      case '3': port = COM3; break;
      case '4': port = COM4; break;
    }
  }
  if( argc >= 3 )
    baud = atol(argv[2]);

#ifdef LINKABLE_TUI
  register_td_tui(tuip);
#else
  if ( !file_exists("TD.TUI") )
  {
    printf("Sorry, cannot find the TD.TUI file");
    exit(0);
  }
  load_tui(tuip, "TD.TUI");
#endif
  init_tui(tuip);
  init_clock(INT_RATE);                         /* use 91 ticks/sec.  This  */
                                                /* is required for tone and */
                                                /* for more precise timing! */
  mk_crc_tbl(0x0000, CRCCITT, gen_crc );
  Term_wnp = tuip->back_wnp;                    /* get desktop window ptr   */
  title_screen(Term_wnp);                       /* do the title screen      */

  if ( init_term_window(Term_wnp, port, baud) ) /* init main Comm screen    */
  {
    termp = (TERM *) Term_wnp->usr_ptr;         /* get attatched terminal   */
    xferp = termp->xferp;                       /* get attached xfer vars   */
    portp = termp->portp;                       /* get attached com port    */

    set_idle_func(term_upd);                    /* let UltraWin call our    */
                                                /* background update func!  */

    modem_cmd("ATE0 S7=60 S11=55 V1 X1 S0=0",   /* send the init string     */
              -1, Tics_per_sec, portp);
    while( !end_flag )
    {
      wait_event();
      /*------------------------- it's a mouse! ----------------------------*/
      if (Event.is_mouse)
      {
        if (Event.m_y == V_rows - 1)            /* click on status line?    */
        {
          if (range(0, Event.m_x, 43))
            term_parms(tuip, TERM_DIALOG);
          else if (range(45, Event.m_x, 64))    /* clicked on com area?     */
            com_parms(tuip, PARMS_DIALOG);
          else if (range(66, Event.m_x, 79))    /* clicked on help area?    */
          {
            tone(1024, 5);
            Csr_in_use = ON;
            tui_help(0, 1, 79, 22, "TD.HLP", "", 0);
            Csr_in_use = OFF;
            pl_csr(Term_wnp);
          }
        }
      } else

      /*------------------------- keyboard event ---------------------------*/
      switch(Event.key)
      {
        case KEY_ALT_C:                         /* clear the window         */
          clear_term_window();
          break;

        case KEY_ALT_E:
          if (xferp->rx_mode == ASCII)
            xferp->abort = ON;
          else
            term_event(Term_wnp);
          break;

        case KEY_ALT_D:
          dial_number(portp);
          break;

        case KEY_ALT_H:                         /* do hangup                */
          set_idle_func(NULL);
          modem_hangup(portp);
          set_idle_func(term_upd);
          break;

        case KEY_ALT_M:                         /* reset the modem          */
          modem_reset(portp);
          break;

        case KEY_ALT_O:                         /* drop to DOS shell        */
          run_dos("shell");
          break;

        case KEY_ALT_P:                         /* do the com setup screen  */
          com_parms(tuip, PARMS_DIALOG);
          break;

        case KEY_ALT_Q:
          if (get_response("Exit EnCom Demo and Hangup?"))
            modem_hangup(portp), end_flag = ON;
          break;

        case KEY_ALT_R:                         /* do the file receive      */
        case KEY_PGDN:
          download_file(tuip, ULDL_DIALOG);
          break;

        case KEY_ALT_S:                         /* do the file send         */
        case KEY_PGUP:
          upload_file(tuip, ULDL_DIALOG);
          break;

        case KEY_ALT_T:                         /* do the term setup screen */
          term_parms(tuip, TERM_DIALOG);
          break;

        case KEY_ALT_X:                         /* quit or exit the demo    */
          if (get_response("Exit EnCom Demo?"))
            end_flag = ON;
          break;

        case KEY_ALT_Z:
          tone(1024, 5);
          Csr_in_use = ON;
          tui_help(0, 1, 79, 22, "TD.HLP", "", 0);
          Csr_in_use = OFF;
          pl_csr(Term_wnp);
          break;

        case 0x1E:                              /* CTRL-C translation       */
          Event.key = KEY_CTRL_C;
          term_event(Term_wnp);
          break;

        default:                                /* pass key to terminal     */
          term_event(Term_wnp);
          break;
      }
    }
    set_idle_func(NULL);
    kill_term_window(Term_wnp);                 /* cleanup com routine      */
  } 

  end_clock();                                  /* restore the clock        */
  end_tui(tuip);                                /* end the TUI              */
  free_tui(tuip);                               /* and free TUI memory      */
  exit(0);
}
/*** end of main ***/

/*****************/
/* ~title_screen */
/*               ************************************************************/
/* This routine does the EnCom title screen, logo and some info.            */
/****************************************************************************/
char  *Title_strs[10] =
{
"                          ",
" ۱     ۱  ۱  ۱۱      ۱  ",
" ۱   ۱ ۱ ۱۱   ۱ ",
" ۱       ۱ ۱ ۱      ۱ ۱ ۱۱۱ ",
"      ۱۱۱ ۱      ۱ ۱ ۱۱۱ ",
" ۱    ۱۱ ۱      ۱ ۱ ۱ ۱ ",
" ۱   ۱ ۱ ۱      ۱ ۱ ۱   ۱ ",
"    ۱  ۱    ۱ ۱    ۱ ",
"                     ",
"                          "
};
char  *Wave_strs[3] =
{
"Ŀ    ڿ              Ŀ Ŀڿ          ",
"                                              ",
"                 "
};
char *Info_strs[7] =
{
"The EnCom Library Terminal Demo 1.60",
"Created with UltraWin, InTUItion and EnCom, (C) 1990-95 EnQue Software",
"EnQue Software",
"25700 East 203rd. Street",
"Pleasant Hill, MO  64080",
"  Phone/Fax: (816) 987-2515",
"24-hour BBS: (816) 566-2956"
};
void title_screen( WINDOW *wnp )
{
  register  int i, r, c, c1 = 7, c2 = (c1 + 8) % 16;
  int       len = strlen(Wave_strs[0]);
  int       x_off = (V_cols - len - 1) / 2;
  int       y_off = 1;
  WINDOW    u_wn, b_wn;
  uchar     b = 0, *mask, *src;
  ulong     old_tics = Tics;

  /*--------------------- setup the windows and such -----------------------*/
  csr_hide();
  m_hide();
  wn_create(x_off, y_off, x_off + 57, y_off + 13, DBL_BDR, WN_POPUP, &b_wn);
  b_wn.att = b_wn.bdr_att = (c1 << 4) | WHITE;
  wn_set( &b_wn);
  wn_create(x_off, y_off, x_off + 57, y_off + 13, NO_BDR, WN_NORMAL, &u_wn);
  u_wn.att = u_wn.bdr_att = (c1 << 4) | c2;
  wn_set( &u_wn);
  wn_border(&b_wn);
  u_wn.scroll = b_wn.scroll = OFF;
  mv_cs(1, 1, &u_wn);
  wn_qch(len, ' ', &u_wn);
  for (i=0; i<10; i++)
    wn_plst(1, i + 2, Title_strs[i], &u_wn);
  mv_cs(1, 12, &u_wn);
  wn_qch(len, ' ', &u_wn);

  /*--------------------- set the mask for the overlay ---------------------*/
  mask = b_wn.mask;
  src =  u_wn.buff;
  for( r = 0; r < b_wn.rows; r++ )
    for( c = 0; c < b_wn.cols; c++, mask++, src+=2 )
      if ( ((*src != ' ') && (*src != 0)) )
        *mask += 1;
  b_wn.mask_on = ON;

  wn_color(WHITE, BLUE, wnp);
  for (i=0; i<2; i++)
    wn_plst(CENTERED, y_off + 15 + i, Info_strs[i], wnp);
  wn_color(YELLOW, BLUE, wnp);
  for (i=2; i<7; i++)
    wn_plst(CENTERED, y_off + 16 + i, Info_strs[i], wnp);

  /*----------------------- do the display loop! ---------------------------*/
  while ( !event_pending() )
  {
    if (Tics > old_tics + 5 )
    {
      old_tics = Tics;
      if ((++b) >= (uchar) len)
      {
        b = 0;
        c1 = (c1 + 1) % 8;
        c2 = (c1 + 8) % 16;
        b_wn.att = b_wn.bdr_att = (c1 << 4) | WHITE;
        wn_border(&b_wn);
        u_wn.att = (c1 << 4) | c2;
        mv_cs(1, 1, &u_wn);
        wn_qch(len, ' ', &u_wn);
        for (i=0; i<10; i++)
          wn_plst(1, i + 2, Title_strs[i], &u_wn);
        mv_cs(1, 12, &u_wn);
        wn_qch(len, ' ', &u_wn);
      }
      mv_cs(0, 0, &b_wn);
      if (len - b)
        wn_st_qty(&Wave_strs[0][b], len - b, &b_wn);
      if (b)
        wn_st_qty(&Wave_strs[0][0], b, &b_wn);
      for (i=0; i<10; i++)
      {
        mv_cs(0, i + 1, &b_wn);
        if (len - b)
          wn_st_qty(&Wave_strs[1][b], len - b, &b_wn);
        if (b)
          wn_st_qty(&Wave_strs[1][0], b, &b_wn);
      }
      mv_cs(0, 11, &b_wn);
      if (len - b)
        wn_st_qty(&Wave_strs[2][b], len - b, &b_wn);
      if (b)
        wn_st_qty(&Wave_strs[2][0], b, &b_wn);
    }
  }
  b_wn.mask_on = OFF;
  wn_destroy(&u_wn);
  wn_destroy(&b_wn);
  m_show();
}
/*** end of title_screen ***/

/*******************/
/* ~clear_selected */
/*                 **********************************************************/
/* Take the items passed by index from start to end, and if they are        */
/* selected turn them off!  This works for buttons, check boxes, etc...     */
/****************************************************************************/
void clear_selected( TUI *tuip, int start, int end )
{
  int i;

  for (i=start; i<=end; i++)
    if (is_selected(tuip, i))
      deselect(tuip, i);
}
/*** end of clear_selected ***/

/**************/
/* ~com_parms */
/*            ***************************************************************/
/* Pull up the com parameters dialog and allow the user to change them.     */
/****************************************************************************/
void com_parms( TUI *tuip, int dlg_inx )
{
  TERM *termp = (TERM *) Term_wnp->usr_ptr;
  PORT *portp = termp->portp;
  int  i, ret_inx, end_flag = OFF;
  int  old_chan, old_parity, old_stop_bits, old_data_bits;
  int  chan, parity, stop_bits, data_bits;
  long old_baud, baud;

  /*------------------------------ get parameters --------------------------*/
  old_chan      = chan      = portp->chan;
  old_parity    = parity    = portp->parity;
  old_data_bits = data_bits = portp->data_bits;
  old_stop_bits = stop_bits = portp->stop_bits;
  old_baud      = baud      = portp->baud;

  /*-------------------------------- set buttons ---------------------------*/
  tone(2048, 5);
  clear_selected(tuip, PARMS_NONE, PARMS_OK);
  for (i=PARMS_BAUD_BEGIN; i<=PARMS_BAUD_END; i++)
    if ( atol((char *) get_btn_text(tuip, i)) == old_baud )
      select(tuip, i);
  switch(old_parity)
  {
    case 'N': select(tuip, PARMS_NONE); break;
    case 'E': select(tuip, PARMS_EVEN); break;
    case 'O': select(tuip, PARMS_ODD);  break;
  }
  switch(old_data_bits)
  {
    case 8: select(tuip, PARMS_DATA8); break;
    case 7: select(tuip, PARMS_DATA7); break;
  }
  switch(old_stop_bits)
  {
    case 1: select(tuip, PARMS_STOP1); break;
    case 2: select(tuip, PARMS_STOP2); break;
  }
  switch(old_chan)
  {
    case COM1: select(tuip, PARMS_COM1); break;
    case COM2: select(tuip, PARMS_COM2); break;
    case COM3: select(tuip, PARMS_COM3); break;
    case COM4: select(tuip, PARMS_COM4); break;
  }

  center_dialog(tuip, dlg_inx);
  draw_dialog(tuip, dlg_inx);
  while (!end_flag)
  {
    ret_inx = do_dialog(tuip, dlg_inx, DO_NORMAL);
    switch( ret_inx )
    {
      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case PARMS_CANCEL:
      case NO_SELECTION:
        end_flag = ON;
        break;

      case PARMS_OK:
        end_flag = ON;
        for (i=PARMS_BAUD_BEGIN; i<=PARMS_BAUD_END; i++)
          if (is_selected(tuip, i))
            baud = atol( (char *) get_btn_text(tuip, i));
        if ( is_selected(tuip, PARMS_NONE) )
          parity = 'N';
        else if ( is_selected(tuip, PARMS_EVEN) )
          parity = 'E';
        else if ( is_selected(tuip, PARMS_ODD) )
          parity = 'O';
        if ( is_selected(tuip, PARMS_DATA8) )
          data_bits = 8;
        else if ( is_selected(tuip, PARMS_DATA7) )
          data_bits = 7;
        if ( is_selected(tuip, PARMS_STOP1) )
          stop_bits = 1;
        else if ( is_selected(tuip, PARMS_STOP2) )
          stop_bits = 2;
        if ( is_selected(tuip, PARMS_COM1) )
          chan = COM1;
        else if ( is_selected(tuip, PARMS_COM2) )
          chan = COM2;
        else if ( is_selected(tuip, PARMS_COM3) )
          chan = COM3;
        else if ( is_selected(tuip, PARMS_COM4) )
          chan = COM4;
        if ( chan != old_chan )
        {
          com_port_destroy(portp);
          if ( com_port_create(chan, baud, parity, data_bits, stop_bits,
                               2048, 2048, portp) < 0 )
          {
            show_alert("Could not create COM port!");
            com_port_create(old_chan, old_baud, old_parity,
                            old_data_bits, old_stop_bits,
                            2048, 2048, portp);
          }
        } else
          com_port_init(baud, parity, data_bits, stop_bits, portp);
        status_line(Term_wnp);
        break;
    }
  }
  erase_dialog(tuip, dlg_inx);
  pl_csr(Term_wnp);
}
/*** end of com_parms ***/

/****************/
/* ~dial_number */
/*              *************************************************************/
/****************************************************************************/
void dial_number( PORT *portp )
{
  TUI   *tuip = &Encom_tui;
  int   dlg_inx = DIAL_DIALOG, ret_inx, end_flag = OFF;
  char  *num_str = (char *) get_inp_text(tuip, DIAL_PHONE);

  tone(2048, 5);
  clear_selected(tuip, DIAL_BBS, DIAL_OK);
  center_dialog(tuip, dlg_inx);
  draw_dialog(tuip, dlg_inx);
  while (!end_flag)
  {
    ret_inx = do_dialog(tuip, dlg_inx, DO_INPUT_ADV);
    switch( ret_inx )
    {
      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case NO_SELECTION:
      case DIAL_CANCEL:
        end_flag = ON;
        break;

      case DIAL_BBS:
        strcpy(num_str, "1-816-566-2956");
        detab(tuip, DIAL_BBS);
        deselect(tuip, DIAL_BBS);
        draw_item(tuip, DIAL_BBS);
        tab(tuip, DIAL_PHONE);
        break;

      case DIAL_OK:
        com_puts((uchar *) "ATDT", portp);
        com_puts((uchar *) num_str, portp);
        com_puts((uchar *) "\r", portp);
        end_flag = ON;
        break;
    }
  }
  erase_dialog(tuip, dlg_inx);
  pl_csr(Term_wnp);
}
/*** end of dial_number ***/

/***************/
/* ~show_alert */
/*             **************************************************************/
/* Take the alert string passed and do a dialog box showing it.             */
/****************************************************************************/
void show_alert( char *alert )
{
  TUI *tuip = &Encom_tui;
  int dlg_inx = ALERT_DIALOG;

  tone(2048, 5);
  set_btn_text(tuip, ALERT_MESSAGE, alert);
  deselect(tuip, ALERT_OK);
  draw_dialog(tuip, dlg_inx);
  do_dialog(tuip, dlg_inx, DO_NORMAL);
  erase_dialog(tuip, dlg_inx);
  pl_csr(Term_wnp);
}
/*** end of show_alert ***/

/*****************/
/* ~get_response */
/*               ************************************************************/
/* Take the questions string passed and do a dialog box with yes and no     */
/* buttons for responses.                                                   */
/****************************************************************************/
int get_response( char *question )
{
  TUI *tuip = &Encom_tui;
  int dlg_inx = ASK_DIALOG, status = 0;

  tone(2048, 5);
  set_btn_text(tuip, ASK_QUESTION, question);
  deselect(tuip, ASK_YES);
  deselect(tuip, ASK_NO);
  draw_dialog(tuip, dlg_inx);
  if (do_dialog(tuip, dlg_inx, DO_NORMAL) == ASK_YES)
    status = 1;
  erase_dialog(tuip, dlg_inx);
  pl_csr(Term_wnp);
  return(status);
}
/*** end of get_response ***/

/****************/
/* ~status_line */
/*              *************************************************************/
/* This function displays a status line at the bottom of the window passed  */
/* by pointer for the port that is attached.                                */
/****************************************************************************/
void status_line( WINDOW *wnp )
{
  TERM *termp = (TERM *) wnp->usr_ptr;
  PORT *portp = termp->portp;
  char status_str[81];

  sprintf(status_str,
    " %s  %s  %s  %s  %6ld,%c,%d,%d COM%d   Alt-Z Help ",
    Term_wnp->cr_lf_flag  ? "CR/LF"     : "CR   ",
    Term_wnp->scroll      ? "SCROLL ON ": "SCROLL OFF",
    Term_wnp->eol_wrap    ? "WRAP ON "  : "WRAP OFF",
       termp->echo        ? "ECHO ON "  : "ECHO OFF",
    portp->baud, portp->parity, portp->data_bits, portp->stop_bits,
    portp->chan + 1);
  msg_line(status_str, Term_wnp);
}
/*** end of status_line ***/

/*************/
/* ~msg_line */
/*           ****************************************************************/
/* This routine clears the bottom line of the window, and outputs the new   */
/* message string, centering it on the line.                                */
/****************************************************************************/
void msg_line( char *msg, WINDOW *wnp )
{
  m_hide();
  push(wnp->att);
  push(wnp->csr_x);
  push(wnp->csr_y);
  wn_color(BLACK, LIGHTGRAY, wnp);
  mv_cs(0, V_rows - 1, wnp);
  wn_claol(wnp);
  wn_plst(CENTERED, V_rows - 1, msg, wnp);
  pop(wnp->csr_y);
  pop(wnp->csr_x);
  pop(wnp->att);
  m_show();
}
/*** end of msg_line ***/

/***********/
/* run_dos */
/*         ******************************************************************/
/*  This routine allows the user to enter a DOS "shell"...                  */
/****************************************************************************/
void run_dos( char *dos_cmd )
{
  int   end_flag = OFF;
  char  cmd[128], dir[128], save_dir[128];

  if( strcmp("shell", dos_cmd) )                  /* run shell or other?    */
    system(dos_cmd);
  else
  {
    m_hide();
    end_clock();
    end_mouse();
    end_video();
    getcwd(save_dir, 128);                        /* save current directory */
    printf("        ۲\n");
    printf("            Type <ENCOM> or <EXIT> to return to the Demo    ۲\n" );
    printf("        ۲\n\n");
    while( !end_flag )
    {
      getcwd(dir, 128);
      printf("\n(EnCom) %s>", dir);
      gets(cmd);
      if (strlen(cmd))
        if( strcmp("EXIT", strupr(cmd)) && strcmp("EnCom", strupr(cmd)) )
          system(cmd);
        else end_flag = ON;
    }
    chdir(save_dir);                              /* restore original dir   */
    init_video(V_cols, V_rows);
    init_mouse();
    init_clock(INT_RATE);
    wn_rfsh(Term_wnp);
    pl_csr(Term_wnp);
    m_show();
  }
}
/*** end of run_dos ***/

#ifdef LINKABLE_TUI
/*------------------- slider/image/palette globals -------------------------*/
uchar Palette_td[16] =
  { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x14, 0x07,
    0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F };

/*----------------------------- prototypes ---------------------------------*/
int  register_td_tui( TUI *tuip );
void register_td_dlg0( TUI *tuip );
void register_td_dlg1( TUI *tuip );
void register_td_dlg2( TUI *tuip );
void register_td_dlg3( TUI *tuip );
void register_td_dlg4( TUI *tuip );
void register_td_dlg5( TUI *tuip );

/********************/
/* ~register_td_tui */
/*                  *********************************************************/
/* Call this function to register the TUI.  This replaces load_tui!         */
/****************************************************************************/
int register_td_tui( TUI *tuip )
{
  Tui_error = OFF;
  setmem(tuip, sizeof(TUI), 0);
  tuip->version_num = 160L;
  tuip->blink_set = ON;
  tuip->manager_used = ON;
  tuip->rows = 25;
  tuip->back_att = 0x1f;
  tuip->back_bdr_att = 0x1f;
  movmem(Palette_td, tuip->palette, 16);

  register_td_dlg0(tuip);
  register_td_dlg1(tuip);
  register_td_dlg2(tuip);
  register_td_dlg3(tuip);
  register_td_dlg4(tuip);
  register_td_dlg5(tuip);
  set_tui_parenting(tuip);
  if (Tui_error)
  {
    free_tui(tuip);
    return(0);
  } else return(1);
}
/*** end of register_td_tui ***/

/*********************/
/* ~register_td_dlg0 */
/*                   ********************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_td_dlg0( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 3, 6, 73, 12, 0x17, 0x17, 2);
  add_box(tuip, 0, 0x0000, 0, 2, 9, 39, 2, 0x10, 0);
  add_box(tuip, 0, 0x0000, 0, 39, 3, 32, 5, 0x10, 0);
  add_box(tuip, 0, 0x0000, 0, 51, 3, 19, 4, 0x70, 1);
  add_box(tuip, 0, 0x0000, 0, 2, 3, 34, 2, 0x10, 0);
  add_box(tuip, 0, 0x0000, 0, 2, 5, 25, 2, 0x10, 0);
  add_box(tuip, 0, 0x0000, 0, 2, 7, 25, 2, 0x10, 0);
  add_button(tuip, 0, 0x0000, 0, 2, 1, 68, 1, 1,
    0x70, 0x70, 0x70, 0x70, 0x70, 0, "Communications Parameters");
  add_button(tuip, 0, 0x0000, 0, 6, 3, 7, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Parity:");
  add_button(tuip, 78, 0x0007, 0, 14, 3, 6, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "None");
  add_button(tuip, 69, 0x0007, 0, 22, 3, 6, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "Even");
  add_button(tuip, 79, 0x0007, 0, 30, 3, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "Odd");
  add_button(tuip, 0, 0x0000, 0, 40, 3, 10, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Baud Rate:");
  add_button(tuip, 0, 0x0000, 0, 40, 4, 10, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "(F1-F8)");
  add_button(tuip, 0, 0x0000, 0, 3, 5, 10, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Data Bits:");
  add_button(tuip, 56, 0x0007, 0, 14, 5, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "8");
  add_button(tuip, 55, 0x0007, 0, 21, 5, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "7");
  add_button(tuip, 315, 0x0007, 0, 51, 3, 9, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "300");
  add_button(tuip, 319, 0x0007, 0, 60, 3, 10, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "19200");
  add_button(tuip, 316, 0x0007, 0, 51, 4, 9, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "1200");
  add_button(tuip, 320, 0x0007, 0, 60, 4, 10, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "38400");
  add_button(tuip, 317, 0x0007, 0, 51, 5, 9, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "2400");
  add_button(tuip, 321, 0x0007, 0, 60, 5, 10, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "57600");
  add_button(tuip, 318, 0x0007, 0, 51, 6, 9, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "9600");
  add_button(tuip, 322, 0x0007, 0, 60, 6, 10, 1, 0,
    0x70, 0x70, 0x07, 0x07, 0x0f, 0, "115200");
  add_button(tuip, 0, 0x0000, 0, 3, 7, 10, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Stop Bits:");
  add_button(tuip, 49, 0x0007, 0, 14, 7, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "1");
  add_button(tuip, 50, 0x0007, 0, 21, 7, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "2");
  add_button(tuip, 0, 0x0000, 0, 3, 9, 10, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Comm Port:");
  add_button(tuip, 0, 0x0000, 0, 3, 10, 10, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "(Alt 1-4)");
  add_button(tuip, 376, 0x0007, 0, 14, 9, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "1");
  add_button(tuip, 377, 0x0007, 0, 21, 9, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "2");
  add_button(tuip, 378, 0x0007, 0, 28, 9, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "3");
  add_button(tuip, 379, 0x0007, 0, 35, 9, 5, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "4");
  add_button(tuip, 67, 0x0017, 0, 44, 9, 12, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 0, "Cancel");
  add_button(tuip, 75, 0x0017, 0, 58, 9, 12, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 1, "Ok");
}
/*** end of register_td_dlg0 ***/

/*********************/
/* ~register_td_dlg1 */
/*                   ********************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_td_dlg1( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 14, 8, 48, 6, 0x17, 0x17, 2);
  add_button(tuip, 0, 0x0000, 0, 2, 1, 44, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Your Question Goes Here");
  add_button(tuip, 78, 0x0017, 0, 4, 3, 16, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 0, "No");
  add_button(tuip, 89, 0x0017, 0, 27, 3, 16, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 0, "Yes");
}
/*** end of register_td_dlg1 ***/

/*********************/
/* ~register_td_dlg2 */
/*                   ********************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_td_dlg2( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 19, 5, 41, 14, 0x17, 0x17, 2);
  add_button(tuip, 0, 0x0000, 0, 3, 1, 35, 1, 1,
    0x70, 0x70, 0x70, 0x70, 0x70, 0, "Terminal Parameters");
  add_check(tuip, 82, 0x0003, 0, 8, 3, 25, 1, 1,
    0x70, 0x7F, 0x7f, 1, 0x07, 5, "[ ] CR/LF Translation");
  add_check(tuip, 83, 0x0003, 0, 8, 5, 25, 1, 1,
    0x70, 0x7F, 0x7f, 1, 0x07, 4, "[ ] Scroll           ");
  add_check(tuip, 76, 0x0003, 0, 8, 7, 25, 1, 1,
    0x70, 0x7F, 0x7f, 1, 0x07, 4, "[ ] Line Wrap        ");
  add_check(tuip, 69, 0x0003, 0, 8, 9, 25, 1, 1,
    0x70, 0x7F, 0x7f, 1, 0x07, 4, "[ ] Echo             ");
  add_button(tuip, 67, 0x0017, 0, 4, 11, 14, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 0, "Cancel");
  add_button(tuip, 75, 0x0017, 0, 23, 11, 13, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 1, "Ok");
}
/*** end of register_td_dlg2 ***/

/*********************/
/* ~register_td_dlg3 */
/*                   ********************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_td_dlg3( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 11, 3, 57, 19, 0x17, 0x17, 2);
  add_box(tuip, 0, 0x0000, 0, 4, 5, 43, 2, 0x10, 0);
  add_box(tuip, 0, 0x0000, 0, 4, 3, 43, 2, 0x10, 0);
  add_box(tuip, 0, 0x0000, 0, 2, 7, 52, 8, 0x70, 1);
  add_button(tuip, 0, 0x0000, 0, 3, 1, 51, 1, 1,
    0x70, 0x70, 0x70, 0x70, 0x70, 0, "Upload or Download Title");
  add_button(tuip, 0, 0x0000, 0, 4, 3, 10, 1, 0,
    0x17, 0x1f, 0x17, 0x1f, 0x17, 0, "File name:");
  add_input(tuip, 70, 0x0003, 0, 15, 3, 29, 1, 1, 0x70, 0x7f, 0x8a,
    "",
    "________________________________________________________________________________",
    "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU");
  add_button(tuip, 0, 0x0000, 0, 5, 5, 9, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Protocol:");
  add_button(tuip, 88, 0x0007, 0, 15, 5, 8, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "Xmodem");
  add_button(tuip, 89, 0x0007, 0, 25, 5, 8, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "Ymodem");
  add_button(tuip, 65, 0x0007, 0, 35, 5, 7, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 0, "ASCII");
  add_button(tuip, 67, 0x0017, 0, 3, 16, 14, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 0, "Cancel");
  add_button(tuip, 75, 0x0017, 0, 41, 16, 13, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 1, "Ok");
}
/*** end of register_td_dlg3 ***/

/*********************/
/* ~register_td_dlg4 */
/*                   ********************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_td_dlg4( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 13, 9, 48, 6, 0x17, 0x17, 2);
  add_button(tuip, 0, 0x0000, 0, 2, 1, 44, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Your Message Goes Here");
  add_button(tuip, 75, 0x0017, 0, 16, 3, 16, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 1, "Ok");
}
/*** end of register_td_dlg4 ***/

/*********************/
/* ~register_td_dlg5 */
/*                   ********************************************************/
/* Internally called function to register one dialog.                       */
/****************************************************************************/
void register_td_dlg5( TUI *tuip )
{
  add_dialog(tuip, 0x0000, 13, 6, 49, 12, 0x17, 0x17, 3);
  add_button(tuip, 0, 0x0000, 0, 3, 1, 43, 1, 1,
    0x70, 0x70, 0x70, 0x70, 0x70, 0, "Simple Dialer");
  add_button(tuip, 69, 0x0013, 0, 16, 4, 29, 1, 1,
    0x70, 0x7f, 0x07, 0x07, 0x0f, 4, "The EnQue 24-hour BBS!");
  add_button(tuip, 0, 0x0000, 0, 3, 6, 12, 1, 0,
    0x17, 0x17, 0x17, 0x17, 0x17, 0, "Dial String:");
  add_input(tuip, 68, 0x0003, 0, 16, 6, 29, 1, 1, 0x70, 0x7f, 0x8a,
    "",
    "________________________________________________________________________________",
    "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU");
  add_button(tuip, 67, 0x0017, 0, 4, 9, 14, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 0, "Cancel");
  add_button(tuip, 75, 0x0017, 0, 30, 9, 14, 1, 1,
    0x30, 0x3f, 0x07, 0x07, 0x0f, 1, "Ok");
}
/*** end of register_td_dlg5 ***/

#endif

/*** END OF FILE ***/

