/****************************************************************************/
/*                                                                          */
/* TD_XFER.C                                                                */
/*                                                                          */
/* These routines make up the file transfer capability of the demo.  Please */
/* note that the file transfer routines were written to be state-driven.    */
/* That means that they can be set as background functions, giving you the  */
/* ability to download or upload files and still go about your business     */
/* doing whatever you wish in your program -- just start and forget!        */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>
#include <io.h>
#include "t.h"                                  /* the main TUI header file */
#include "encom.h"                              /* the Encom header file    */

#include "td.h"                                 /* our mini-app header file */
#include "td.def"                               /* the TUICP define file    */
#include "td_globx.h"                           /* externs for the app      */

#ifdef __ZTC__
long filelength( int fid );
#endif

/*******************/
/* ~finished_sound */
/*                 **********************************************************/
/****************************************************************************/
void finished_sound( void )
{
  int i;

  for (i=0; i<5; i++)
  {
    tone(2000 + i * 128, 10);
    Uw_timers[0] = 10;
    while (Uw_timers[0] > 0) {}
    sound_off();
    Uw_timers[0] = 3;
    while (Uw_timers[0] > 0) {}
  }
}
/*** end of finished_sound ***/

/*****************/
/* ~ascii_upload */
/*               ************************************************************/
/* Send ASCII function.  Note this function is state driven, and can be     */
/* setup as a background function that gets called as often as you wish!    */
/****************************************************************************/
void ascii_upload( TERM *termp, char *fname )
{
  int     stat;
  PORT    *portp = termp->portp;                /* get attatched port       */
  XFER    *xferp = termp->xferp;                /* get attached xfer vars   */

  switch( xferp->tx_state )
  {
    case 0:
      setmem(xferp, sizeof(XFER), 0);
      strcpy(xferp->fname, fname);
      if( (xferp->tx_fp = fopen(xferp->fname, "r+b")) != NULL )
      {
        xferp->tx_mode = ASCII;
        xferp->tx_state = 1;
        portp->tx_chars = 0;
        xferp->abort = 0;
      }else
        xferp->tx_state = 2;
      break;
    case 1:
      if( xferp->abort )
        xferp->tx_state = 2;
      else
      if( portp->tx_cnt < (portp->tx_size / 8) )
      {
        stat = getc(xferp->tx_fp);
        if (stat == EOF)
          xferp->tx_state = 2;
        else
        {
          if (stat == 0x0A)                     /* pause on linefeeds       */
            wait_ticks(9L);
          xferp->tx_chars++;
          com_putc(stat, portp);
        }
      }
      break;
    case 2:
      if( xferp->tx_fp != NULL )
        fclose(xferp->tx_fp);
      xferp->tx_mode = 0;
      xferp->tx_state = 0;
      xferp->status = 0;
      finished_sound();
      break;
  }
}
/*** end of ascii_upload ***/

/*******************/
/* ~ascii_download */
/*                 **********************************************************/
/****************************************************************************/
void ascii_download( TERM *termp, char *fname )
{
  int   i, j, c;
  char  buff[81];
  PORT  *portp = termp->portp;                  /* get attatched port       */
  XFER  *xferp = termp->xferp;                  /* get attached xfer vars   */

  switch( xferp->rx_state )
  {
    case 0:
      setmem(xferp, sizeof(XFER), 0);
      strcpy(xferp->fname, fname);
      if( (xferp->rx_fp = fopen(xferp->fname, "a+b")) != NULL )
      {
        xferp->rx_mode = ASCII;
        xferp->rx_state = 1;
        xferp->rx_chars = 0;
        xferp->abort = 0;
      }else
        xferp->rx_state = 2;
      break;
    case 1:
      if( xferp->abort )
        xferp->rx_state = 2;
      else
      {
        for(i=0; (i<80) && portp->rx_cnt; i++ )       /* read rx data       */
        {
          c = com_getc(portp);
          if( c == CTRL_Z )                           /* end if CTRL_Z      */
          {
            fwrite(buff, 1, i, xferp->rx_fp);         /* write to file      */
            xferp->rx_state = 2;
          }
          buff[i] = (uchar) c;                        /* save char in buff  */
        }
        if( i )                                       /* if data read...    */
        {
          m_hide();
          for (j=0; j<i; j++)
            (*(termp->vid_func))(buff[j], Term_wnp);
          if ( !Csr_in_use )
            pl_csr(Term_wnp);
          m_show();
          fwrite(buff, 1, i, xferp->rx_fp);           /* ...write to file   */
        }
        xferp->rx_chars += i;
      }
      break;
    case 2:
      if( xferp->rx_fp != NULL )
        fclose(xferp->rx_fp);
      xferp->rx_mode = 0;
      xferp->rx_state = 0;
      xferp->status = 0;
      status_line(Term_wnp);
      finished_sound();
      break;
  }
}
/*** end of ascii_download ***/

/*****************/
/* ~ascii_status */
/*               ************************************************************/
/****************************************************************************/
void ascii_status( TERM *termp, WINDOW *wnp )
{
  PORT  *portp = termp->portp;                  /* get attatched port       */
  XFER  *xferp = termp->xferp;                  /* get attached xfer vars   */

  mv_cs(8, 7, wnp), wn_printf( wnp, "Chars Transmitted: %12lu", xferp->tx_chars    );
  mv_cs(8,10, wnp), wn_printf( wnp, "    Parity Errors: %12d",  portp->parity_errs );
  mv_cs(8,11, wnp), wn_printf( wnp, "   Framing Errors: %12d",  portp->framing_errs);
  mv_cs(8,12, wnp), wn_printf( wnp, "   Overrun Errors: %12d",  portp->overrun_errs);
}
/*** end of ascii_status ***/


/*---------------------------- X/Y modem routines --------------------------*/
/***************/
/* ~save_xpkts */
/*             **************************************************************/
/****************************************************************************/
int save_xpkts( FILE *fp, XFER *xferp )
{
  uint  i;
  uchar *p;

  p = xferp->pkts;
  for( i=0; i<xferp->pkt_curr; i++, p+=xferp->pkt_size )
    if( fwrite(p, xferp->pkt_sizes[i], 1, fp) == 0 )
      return(0);
  return(i);
}
/*** end of save_xpkts ***/

/***************/
/* ~make_xpkts */
/*             **************************************************************/
/****************************************************************************/
int make_xpkts( FILE *fp, XFER *xferp )
{
  int bytes, pkts;

  bytes = fread(xferp->pkts, 1, xferp->num_pkts * xferp->pkt_size, fp);
  pkts = bytes / xferp->pkt_size;
  if( bytes % xferp->pkt_size)
    pkts++;
  return(pkts);
}
/*** end of make_xpkts ***/

/***************/
/* ~calc_cksum */
/*             **************************************************************/
/****************************************************************************/
uchar calc_cksum( uchar *p, int size )
{
  register unsigned short int cksum = 0, qty = size;
  while( qty-- >= 1 )
    cksum += *p++;
  return( (uchar) (cksum % 256) );
}
/*** end of calc_cksum ***/

/**********************/
/* ~xy_modem_download */
/*                    *******************************************************/
/* Send X or YMODEM function.  The rx_mode member of the passed terminal    */
/* structure pointer signifies X or Y.  Note this function is state driven, */
/* and can be setup as a background function that gets called as often as   */
/* you wish!                                                                */
/****************************************************************************/
void xy_modem_download( TERM *termp, int mode, char *fname )
{
  ulong timeout;
  int   i, n, c;
  PORT  *portp = termp->portp;                  /* get attatched port       */
  XFER  *xferp = termp->xferp;                  /* get attached xfer vars   */
  uchar *p;
  uint  pkt_no, not_pkt_no, size;
  unsigned short int ck1, ck2;

  switch( xferp->rx_state )
  {
    /*----------------------------------------------------------------------*/
    case 0:                                           /* open the file      */
      setmem(xferp, sizeof(XFER), 0);
      strcpy(xferp->fname, fname);
      if( (xferp->rx_fp = fopen(xferp->fname, "w+b")) != NULL )
      {
        if( mode == XMODEM )
          xferp->pkt_size = XMODEM_PKT_SZ;
        else
          xferp->pkt_size = YMODEM_PKT_SZ;
        xferp->num_pkts = (XFER_BUFF_SIZE / xferp->pkt_size);
        if( (xferp->pkts = (uchar *) calloc(1, XFER_BUFF_SIZE)) != NULL )
        {
          xferp->rx_tx_flag = 1;
          xferp->rx_state = 1;
          xferp->pkt_no = 1;
          xferp->sync_char = 'C';
          xferp->status = 1;
        } else xferp->rx_state = 100;
      } else xferp->rx_state = 100;
      break;

    /*----------------------------------------------------------------------*/
    case 1:                                           /* initialize sync    */
      if( !xferp->sync_flag )                         /* only for startup!  */
      {
        if( xferp->sync_cnt++ >= SYNC_RETRY )
        {
          xferp->sync_cnt = 0;
          xferp->sync_char = (xferp->sync_char == (uchar) 'C') ? (uchar) NAK : (uchar) 'C';
        }
      }else
        xferp->sync_char = NAK;
      com_putc(xferp->sync_char, portp);
      xferp->xtime = Tics;
      xferp->rx_state = 2;
      break;

    /*----------------------------------------------------------------------*/
    case 2:                                           /* wait for SOH       */
      if (xferp->sync_flag)
        timeout = (ulong) (Tics_per_sec * 10);
      else
        timeout = (ulong) Tics_per_sec;
      if( xferp->abort )                              /* user aborted       */
      {
        xferp->rx_state = 100;
        xferp->msg = X_ABORT;
        for( i = 0; i < 8; i++ )
          com_putc(CAN, portp);
      }
      else if( (Tics - xferp->xtime) >= timeout )     /* if timeout, resend */
      {
        if( xferp->sync_flag )                        /* only err if started*/
        {
          xferp->err_cnt++;
          xferp->msg = X_NOSOH;
        }
        xferp->rx_state = 1;
      }
      else if( portp->rx_cnt )                        /* have any data yet? */
      {
        if( !xferp->pkts_mvd )
          xferp->stime = Tics;
        c = com_getc(portp);
        switch( c )
        {
          case SOH:                                   /* SOH, go get packet */
          case STX:                                   /* 1K packet          */
            if( !xferp->sync_flag )
            {
              xferp->ck_type = (xferp->sync_char == (uchar) 'C') ? (uchar) X_CRC : (uchar) X_CKSUM;
              xferp->sync_flag = 1;
            }
            xferp->rx_state = 3;
            xferp->xtime = Tics;
            xferp->pkt_sizes[xferp->pkt_curr] = (c == STX) ? YMODEM_PKT_SZ : XMODEM_PKT_SZ;
            break;
          case EOT:                                   /* rx successfull     */
            xferp->msg = X_OK;
            save_xpkts(xferp->rx_fp, xferp);
            if( ferror(xferp->rx_fp) )
            {
              xferp->err_cnt++;
              xferp->rx_state = 100;
              xferp->msg = X_FWRITE;
              for( i = 0; i < 8; i++ )
                com_putc(CAN, portp);
            }else
              com_putc(ACK, portp);
            xferp->rx_state = 100;
            break;
          case CAN:                                   /* tx'r cancelled xfer*/
            xferp->err_cnt++;
            xferp->msg = X_CAN;
            xferp->rx_state = 100;
            break;
          default:                                    /* bad SOH character  */
            xferp->err_cnt++;
            com_clear_que(RX, portp);
            xferp->rx_state = 1;
            xferp->msg = X_BADSOH;
            break;
        }
      }
      break;

    /*----------------------------------------------------------------------*/
    case 3:                                           /* SOH OK, get packet */
      size = xferp->pkt_sizes[xferp->pkt_curr];
      if( ( Tics - xferp->xtime ) >= (ulong) (Tics_per_sec * 10) )
      {
        xferp->msg = X_NODATA;
        xferp->rx_state = 1;
      }
      else if( portp->rx_cnt >= size + ((xferp->ck_type == X_CRC) ? 4:3) )
      {                                               /* entire pkt here?   */
        n = xferp->pkt_curr;
        p = xferp->pkts + (n * xferp->pkt_size);
        pkt_no = com_getc(portp);
        not_pkt_no = com_getc(portp);
        com_getc_qty(p, size, portp);
        if( xferp->ck_type == X_CKSUM )
        {
          ck1 = calc_cksum(p, size);
          ck2 = com_getc(portp);
        }else{
          ck1 = calc_crc(p, size);
          ck2  = com_getc(portp);
          ck2 <<= 8;
          ck2 |= com_getc(portp);
        }
        if( ck1 != ck2 )                              /* is check val OK?     */
        {
          xferp->msg = (xferp->ck_type == X_CKSUM) ? X_CK_ERR : X_CRC_ERR;
          xferp->rx_state = 1;
        }
        else if( pkt_no != ((~not_pkt_no) & 0xff) )     /* is packet no OK?   */
        {
          xferp->msg = X_PKTNUM;
          xferp->rx_state = 1;
        }
        else if( pkt_no != (uint) ((xferp->pkts_mvd + 1L) % 256L) )/* seq OK? */
        {
          if( pkt_no == (uint) (xferp->pkts_mvd % 256L) )/* packet repeat OK  */
          {
            xferp->msg = X_DUPPKT;
            com_putc(ACK, portp);
            xferp->xtime = Tics;
            xferp->rx_state = 2;
          }else{                                      /* otherwise, FATAL   */
            xferp->msg = X_PKTSEQ;
            for( i = 0; i < 8; i++ )
              com_putc(CAN, portp);
            xferp->rx_state = 100;
          }
        }else{
          xferp->pkts_mvd++;
          xferp->bytes_mvd += size;
          xferp->pkt_no = (xferp->pkt_no + 1) % 256;
          xferp->pkt_curr++;
          if( xferp->pkt_curr >= xferp->num_pkts )
          {
            save_xpkts(xferp->rx_fp, xferp);
            if( ferror(xferp->rx_fp) )
            {
              xferp->rx_state = 100;
              xferp->msg = X_FWRITE;
              for( i = 0; i < 8; i++ )
                com_putc(CAN, portp);
            }
            xferp->pkt_curr = 0;
          }
          if( xferp->rx_state != 100 )                      /* disk write OK?  */
          {
            com_putc(ACK, portp);
            xferp->xtime = Tics;
            xferp->rx_state = 2;
          }
        }
      }
      break;

    /*-------------------- this state cleans everything up -----------------*/
    case 100:
      xferp->rx_state = 101;
      break;
    case 101:
      if( xferp->rx_fp != NULL )
      {
        fclose(xferp->rx_fp);
        xferp->rx_fp = NULL;
      }
      if( xferp->pkts != NULL )
        free(xferp->pkts);
      xferp->rx_mode = xferp->tx_mode = 0;
      xferp->rx_state = 0;
      xferp->status = 0;
      xferp->rx_tx_flag = 0;
      finished_sound();
      break;
  }
}
/*** end of xy_modem_download ***/

/********************/
/* ~xy_modem_upload */
/*                  *********************************************************/
/****************************************************************************/
void xy_modem_upload( TERM *termp, int mode, char *fname )
{
  int   i, n, c;
  PORT  *portp = termp->portp;                  /* get attatched port       */
  XFER  *xferp = termp->xferp;                  /* get attached xfer vars   */
  uchar *p, cksum;
  unsigned short int crc;

  switch( xferp->tx_state )
  {
    /*----------------------------------------------------------------------*/
    case 0:                                           /* open the file      */
      setmem(xferp, sizeof(XFER), 0);
      strcpy(xferp->fname, fname);
      if( (xferp->tx_fp = fopen(xferp->fname, "r+b")) != NULL )
      {
        if( mode == XMODEM )
          xferp->pkt_size = XMODEM_PKT_SZ;
        else
          xferp->pkt_size = YMODEM_PKT_SZ;
        xferp->num_pkts = (XFER_BUFF_SIZE / xferp->pkt_size);
        if( (xferp->pkts = (uchar *) calloc(1, XFER_BUFF_SIZE)) != NULL )
        {
          xferp->abort = 0;
          xferp->rx_tx_flag = 2;
          xferp->tx_state = 1;
          xferp->pkt_no = 1;
          xferp->status = 1;
          xferp->fsize = filelength(fileno(xferp->tx_fp));
          xferp->tot_pkts = xferp->fsize / xferp->pkt_size;
          if( xferp->fsize % xferp->pkt_size )
            xferp->tot_pkts++;
          com_clear_que(RX, portp);
          if( (xferp->pkt_cnt = make_xpkts(xferp->tx_fp, xferp)) == 0 )
          {
            xferp->tx_state = 100;
            xferp->msg = X_FEMPTY;
          }
          if( ferror(xferp->tx_fp) )
          {
            xferp->tx_state = 100;
            xferp->msg = X_FREAD;
          }
        } else xferp->tx_state = 100;
      } else xferp->tx_state = 100;
      break;

    /*----------------------------------------------------------------------*/
    case 1:                                           /* hunt for sync char */
      if( xferp->abort )                              /* user aborted       */
      {
        xferp->tx_state = 100;
        xferp->msg = X_ABORT;
        for( i = 0; i < 8; i++ )
          com_putc(CAN, portp);
      }
      else if( portp->rx_cnt )
      {
        c = com_getc(portp);
        switch( c )
        {
          case 'C':                                   /* rx'r wants crc     */
            xferp->ck_type = X_CRC;
            xferp->stime = Tics;
            xferp->tx_state = 2;
            break;
          case NAK:                                   /* rx'r wants cksum   */
            xferp->stime = Tics;
            xferp->ck_type = X_CKSUM;
            xferp->tx_state = 2;
            break;
          case CAN:                                   /* rx'r cancelled xfer*/
            xferp->err_cnt++;
            xferp->tx_state = 100;
            xferp->msg = X_CAN;
            break;
        }
      }
      break;

    /*----------------------------------------------------------------------*/
    case 2:                                           /* send packet        */
      if( xferp->pkts_mvd < xferp->tot_pkts )
      {
        n = xferp->pkt_curr;
        p = xferp->pkts + (n * xferp->pkt_size);
        if( xferp->pkt_size == YMODEM_PKT_SZ )
          com_putc(STX, portp);
        else
          com_putc(SOH, portp);
        com_putc(xferp->pkt_no, portp);
        com_putc(~xferp->pkt_no, portp);
        com_putc_qty(p, xferp->pkt_size, portp);
        if( xferp->ck_type == X_CKSUM )
        {
          cksum = calc_cksum(p, xferp->pkt_size);
          com_putc(cksum, portp);
        } else
        {
          crc = calc_crc(p, xferp->pkt_size);
          com_putc(hibyte(crc), portp);
          com_putc(lobyte(crc), portp);
        }
        xferp->tx_state = 3;
        xferp->xtime = Tics;                          /* mark time of send  */
      } else xferp->tx_state = 4;                     /* last packet sent   */
      break;

    /*----------------------------------------------------------------------*/
    case 3:                                           /* wait for response  */
      if( xferp->abort )                              /* user aborted       */
      {
        xferp->tx_state = 100;
        xferp->msg = X_ABORT;
        for( i = 0; i < 8; i++ )
          com_putc(CAN, portp);
      } else if ( (Tics - xferp->xtime) >= (ulong) (Tics_per_sec * 30) )
      {
        xferp->tx_state = 100;
        xferp->msg = X_NOACK;
      }
      else if( portp->rx_cnt )                        /* any data received  */
      {
        c = com_getc(portp);
        switch( c )
        {
          case ACK:                                   /* packet ok          */
            xferp->pkt_no = (xferp->pkt_no + 1) % 256;
            xferp->pkts_mvd++;
            xferp->bytes_mvd += xferp->pkt_size;
            xferp->pkt_curr++;
            if( xferp->pkt_curr >= xferp->pkt_cnt )
            {
              xferp->pkt_cnt = make_xpkts(xferp->tx_fp, xferp);
              xferp->pkt_curr = 0;
            }
            xferp->resend = 0;
            xferp->tx_state  = 2;
            break;
          case NAK:                                   /* resend packet      */
            com_clear_que(RX, portp);
            xferp->err_cnt++;
            if( ++xferp->resend >= ACK_RETRY )
            {
              xferp->tx_state = 100;
              xferp->msg = X_NOACK;
            } else
            {
              xferp->tx_state = 2;
              xferp->msg = X_NAK;
            }
            break;
          case CAN:                                   /* rx'r cancelled xfer*/
            xferp->err_cnt++;
            xferp->tx_state = 100;
            xferp->msg = X_CAN;
            break;
        }
      }
      break;

    /*----------------------------------------------------------------------*/
    case 4:                                           /* send EOT           */
      com_putc(EOT, portp);
      xferp->xtime = Tics;
      xferp->tx_state = 5;
      break;

    /*----------------------------------------------------------------------*/
    case 5:                                           /* wait for final ACK */
      if( xferp->abort )                              /* user aborted       */
      {
        xferp->tx_state = 100;
        xferp->msg = X_ABORT;
        for( i = 0; i < 8; i++ )
          com_putc(CAN, portp);
      }
      else if ( (Tics - xferp->xtime) >= (ulong) (Tics_per_sec * 2) )
      {
        if( ++xferp->resend >= ACK_RETRY )
        {
          com_putc(CAN, portp);
          xferp->tx_state = 100;
          xferp->msg = X_NOEOT;
        }else
          xferp->tx_state = 4;
      }
      else if( portp->rx_cnt )
      {
        c = com_getc(portp);
        switch( c )
        {
          case ACK:                                   /* tx successfull     */
            xferp->tx_state = 100;
            xferp->msg = X_OK;
            break;
          case CAN:                                   /* rx'r cancelled xfer*/
            xferp->err_cnt++;
            xferp->tx_state  = 100;
            xferp->msg = X_CAN;
            break;
        }
      }
      break;

    /*------------------ this routine cleans everything up -----------------*/
    case 100:
      xferp->tx_state = 101;
      break;
    case 101:
      if( xferp->tx_fp != NULL )
      {
        fclose(xferp->tx_fp);
        xferp->tx_fp = NULL;
      }
      if( xferp->pkts != NULL )
        free(xferp->pkts);
      xferp->rx_mode = xferp->tx_mode = 0;
      xferp->tx_state = 0;
      xferp->rx_tx_flag = 0;
      xferp->status = 0;
      finished_sound();
      break;
  }
}
/*** end of xy_modem_upload ***/

/********************/
/* ~xy_modem_status */
/*                  *********************************************************/
/****************************************************************************/
void xy_modem_status( TERM *termp, WINDOW *wnp, int mode )
{
  static ulong last_tics = 0L;
  XFER  *xferp = termp->xferp;
  ulong secs, cps, bytes;

  /*------------- output these statistics only on the first call -----------*/
  if( !mode )
  {
    wn_plst(4, 6, "Block Check   :", wnp );
    wn_plst(4, 7, "Elapsed Time  :", wnp );
    wn_plst(4, 8, "Time Remaining:", wnp );
    wn_plst(4, 9, "Bytes Xferred :", wnp );
    wn_plst(4,10, "Blocks Xferred:", wnp );
    wn_plst(4,11, "Chars/Sec     :", wnp );
    wn_plst(4,12, "Error Count   :", wnp );
    wn_plst(4,13, "Status        :", wnp );
  } else
  {
    /*----------------- output these statistics once a second ----------------*/
    bytes = xferp->bytes_mvd;
    if( (Tics - last_tics) > (ulong) Tics_per_sec )
    {
      last_tics = Tics;
      wn_plst(20, 6, (xferp->ck_type == X_CRC) ? "CRC     ":"CHECKSUM", wnp);
      if( xferp->stime )                               /* wait until start  */
      {
        secs = max( 1L, (Tics - xferp->stime) / (ulong) Tics_per_sec);
        cps = max( 1L, bytes / secs );
        mv_cs(20, 7, wnp), wn_printf(wnp, "%02lu:%02lu", secs / 60L, secs % 60L);
        if( xferp->rx_tx_flag == 2&& (secs >= 2L) )   /* only display on TX */
        {
          if( xferp->fsize > bytes )                  /* calc if not done   */
            secs = (ulong) ( (xferp->fsize - bytes) / cps );
          else
            secs = 0L;
          mv_cs(20, 8, wnp), wn_printf(wnp, "%02lu:%02lu", secs / 60L, secs % 60L);
        }
        mv_cs(20, 11, wnp), wn_printf(wnp, "%-5lu", cps);
      }
      mv_cs(20, 12, wnp), wn_printf(wnp, "%-5d",  xferp->err_cnt);
      mv_cs(20, 13, wnp), wn_printf(wnp, "%-16s", X_errs[xferp->msg]);
    }

    /*----------------- output these statistics every time ---------------------*/
    mv_cs(20,  9, wnp);
    if (xferp->fsize)
      wn_printf( wnp, "%lu of %lu", bytes, xferp->fsize);
    else
      wn_printf( wnp, "%lu ", bytes);
    mv_cs(20, 10, wnp);
    if (xferp->tot_pkts)
      wn_printf( wnp, "%lu of %lu", xferp->pkts_mvd, xferp->tot_pkts);
    else
      wn_printf( wnp, "%lu ", xferp->pkts_mvd);
  }
}
/*** end of xy_modem_status ***/

/****************/
/* ~upload_file */
/*              *************************************************************/
/****************************************************************************/
void upload_file( TUI *tuip, int dlg_inx )
{
  int     ret_inx, mode, end_flag = OFF;
  int     data_bits = 0, parity;
  char    buffer[81];
  char    *fname = (char *) get_inp_text(tuip, ULDL_FNAME);
  TERM    *termp = (TERM *) Term_wnp->usr_ptr;    /* get attatched terminal */
  PORT    *portp = termp->portp;                  /* get attatched port     */
  XFER    *xferp = termp->xferp;                  /* get attached xfer vars */
  WINDOW  *wnp;                                   /* win ptr for the dlg    */

  set_str_text(tuip, ULDL_TITLE, "Send File");    /* init some dlg items    */
  deselect(tuip,  ULDL_CANCEL);
  deselect(tuip,  ULDL_OK);
  if ( !is_selected(tuip, ULDL_XMODEM)
    && !is_selected(tuip, ULDL_YMODEM)
      && !is_selected(tuip, ULDL_ASCII) )
    select(tuip, ULDL_XMODEM);

  center_dialog(tuip, dlg_inx);                   /* center and draw dlg    */
  draw_dialog(tuip, dlg_inx);
  wnp = get_dlg_wnp(tuip, dlg_inx);               /* get dialog window ptr  */
  while (!end_flag)
  {
    ret_inx = do_dialog(tuip, dlg_inx, DO_INPUT_ADV);
    switch( ret_inx )
    {
      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case NO_SELECTION:
      case ULDL_CANCEL:
        detab(tuip, ULDL_CANCEL);
        status_line(Term_wnp);
        end_flag = ON;
        break;

      /*----------------------- ok pressed, try upload ---------------------*/
      case ULDL_OK:
        if ( file_exists(fname) )
        {
          sprintf(buffer, "Uploading %s, any key to abort", fname);
          msg_line(buffer, Term_wnp);
          push(wnp->att);
          draw_item(tuip, ULDL_BOX);
          wnp->att = get_box_att(tuip, ULDL_BOX);
          portp->parity_errs = portp->framing_errs = 0;
          portp->overrun_errs = 0;

          /*--------------------------- do ASCII -----------------------------*/
          if ( is_selected(tuip, ULDL_ASCII) )
          {
            do
            {
              ascii_upload(termp, fname);
              ascii_status(termp, wnp);
              term_upd();
              if (event_pending())
                xferp->abort = ON;
            } while (xferp->tx_state);
          } else

          /*-------------------------- do X/Y modem --------------------------*/
          if ( is_selected(tuip, ULDL_XMODEM) ||
            is_selected(tuip, ULDL_YMODEM) )
          {
            if ( is_selected(tuip, ULDL_XMODEM) )
              mode = XMODEM;
            else
              mode = YMODEM;
            if (portp->data_bits == 7)              /* switch to 8 data bits  */
            {                                       /* if necessary!          */
              data_bits = portp->data_bits;
              parity    = portp->parity;
              com_port_init(portp->baud, 'N', 8,
                            portp->stop_bits, portp);
            }
            xy_modem_status(termp, wnp, 0);         /* do field names         */
            do
            {
              xy_modem_upload(termp, mode, fname);  /* do upload              */
              xy_modem_status(termp, wnp, 1);       /* show the status        */
              if (event_pending())                  /* abort if any event     */
                xferp->abort = ON;
            } while (xferp->tx_state);

            if (data_bits)                          /* switch back?           */
              com_port_init(portp->baud, parity, data_bits,
                            portp->stop_bits, portp);
          }

          pop(wnp->att);
          deselect(tuip, ULDL_OK);
          detab(tuip, ULDL_OK);
          draw_item(tuip, ULDL_OK);
          status_line(Term_wnp);
        } else
        {
          tone(2048, 10);
          sprintf(buffer, "%s does not exist!", fname);
          msg_line(buffer, Term_wnp);
          strcpy(fname, "\0");
          deselect(tuip, ULDL_OK);
          detab(tuip, ULDL_OK);
          draw_item(tuip, ULDL_OK);
          tab(tuip, ULDL_FNAME);
        }
        break;
    }
  }
  erase_dialog(tuip, dlg_inx);
  pl_csr(Term_wnp);
}
/*** end of upload_file ***/

/******************/
/* ~download_file */
/*                ***********************************************************/
/****************************************************************************/
void download_file( TUI *tuip, int dlg_inx )
{
  int     ret_inx, mode, end_flag = OFF;
  int     data_bits = 0, parity;
  char    buffer[81];
  char    *fname = (char *) get_inp_text(tuip, ULDL_FNAME);
  TERM    *termp = (TERM *) Term_wnp->usr_ptr;    /* get attatched terminal */
  PORT    *portp = termp->portp;                  /* get attatched port     */
  XFER    *xferp = termp->xferp;                  /* get attached xfer vars */
  WINDOW  *wnp;                                   /* win ptr for the dlg    */

  set_str_text(tuip, ULDL_TITLE, "Receive File"); /* init some dlg items    */
  deselect(tuip,  ULDL_CANCEL);
  deselect(tuip,  ULDL_OK);
  if ( !is_selected(tuip, ULDL_XMODEM)
    && !is_selected(tuip, ULDL_YMODEM)
      && !is_selected(tuip, ULDL_ASCII) )
    select(tuip, ULDL_XMODEM);

  center_dialog(tuip, dlg_inx);                   /* center and draw dlg    */
  draw_dialog(tuip, dlg_inx);
  wnp = get_dlg_wnp(tuip, dlg_inx);               /* get dialog window ptr  */
  while (!end_flag)
  {
    ret_inx = do_dialog(tuip, dlg_inx, DO_INPUT_ADV);
    switch( ret_inx )
    {
      /*---------------- ESC pressed or clicked off dialog -----------------*/
      case NO_SELECTION:
      case ULDL_CANCEL:
        detab(tuip, ULDL_CANCEL);
        status_line(Term_wnp);
        end_flag = ON;
        break;

      /*----------------------- ok pressed, try download -------------------*/
      case ULDL_OK:
        push(wnp->att);
        draw_item(tuip, ULDL_BOX);
        wnp->att = get_box_att(tuip, ULDL_BOX);
        portp->parity_errs = portp->framing_errs = 0;
        portp->overrun_errs = 0;

        if ( is_selected(tuip, ULDL_ASCII) )
        {
          sprintf(buffer, "ASCII downloading to %s, press ALT-E to end", fname);
          msg_line(buffer, Term_wnp);
          ascii_download(termp, fname);
          end_flag = ON;
        } else
        if ( is_selected(tuip, ULDL_XMODEM) ||
          is_selected(tuip, ULDL_YMODEM) )
        {
          if ( is_selected(tuip, ULDL_XMODEM) )
            mode = XMODEM;
          else
            mode = YMODEM;
          if (portp->data_bits == 7)              /* switch to 8 data bits  */
          {                                       /* if necessary!          */
            data_bits = portp->data_bits;
            parity    = portp->parity;
            com_port_init(portp->baud, 'N', 8,
                          portp->stop_bits, portp);
          }
          xy_modem_status(termp, wnp, 0);         /* do field names         */
          do
          {
            xy_modem_download(termp, mode, fname);/* do download            */
            xy_modem_status(termp, wnp, 1);       /* show the status        */
            if (event_pending())                  /* abort if any event     */
              xferp->abort = ON;
          } while (xferp->rx_state);

          if (data_bits)                          /* switch back?           */
            com_port_init(portp->baud, parity, data_bits,
                          portp->stop_bits, portp);
        }

        pop(wnp->att);
        deselect(tuip, ULDL_OK);
        detab(tuip, ULDL_OK);
        draw_item(tuip, ULDL_OK);
        break;
    }
  }
  erase_dialog(tuip, dlg_inx);
  pl_csr(Term_wnp);
}
/*** end of download_file ***/

#ifdef __ZTC__
long filelength( int fid )
{
  long save, len;
  save = lseek(fid, 0L, SEEK_CUR);
  len = lseek(fid, 0L, SEEK_END);
  lseek(fid, save, SEEK_SET);
  return(len);
}
#endif

/*** END OF FILE ***/
